/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrablockcommand.h"

TundraBlockCommand::TundraBlockCommand(TundraModel* m, QPoint& tl, QPoint& br, TundraBlockStrategy* s) : is_remote(false)
{
    model = m;
    strategy = s;
    top_left = tl;
    bottom_right = br;

    width = br.x()-tl.x();
    height = br.y()-tl.y();

    doc_block = new TundraDoc*[height];
    doc_block[0] = new TundraDoc[width*height];

    for(int r = 1; r < height; r++)
        doc_block[r] = doc_block[0] + (width*r);
}

TundraBlockCommand::~TundraBlockCommand(void)
{
    delete [] doc_block[0];
    delete [] doc_block;
}

void TundraBlockCommand::execute() {
    TundraDoc* current;

    // Store the old area
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            if(((top_left.x()+x) <= model->getMaxX()) && (top_left.x()+x) >= 0 && (top_left.y()+y) >= 0 && ((top_left.y()+y) <= model->getMaxY())) {
                current = model->getItem(top_left.x()+x, top_left.y()+y);

                doc_block[y][x].character = current->character;
                doc_block[y][x].fg_color = current->fg_color;
                doc_block[y][x].bg_color = current->bg_color;
            }
        }
    }

    strategy->execute(model, top_left, bottom_right);

    model->setModified();
}

void TundraBlockCommand::unexecute() {

    TundraDoc* current;

    // Restore the old block
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            if(((top_left.x()+x) <= model->getMaxX()) && (top_left.x()+x) >= 0 && (top_left.y()+y) >= 0 && ((top_left.y()+y) <= model->getMaxY())) {
                current = model->getItem(top_left.x()+x, top_left.y()+y);

                current->character = doc_block[y][x].character;
                current->fg_color = doc_block[y][x].fg_color;
                current->bg_color = doc_block[y][x].bg_color;
            }
        }
    }

    model->setModified();
}

