/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRABLOCKCOMMAND_H
#define TUNDRABLOCKCOMMAND_H

#include <qpoint.h>

#include "tundrablockstrategy.h"
#include "tundracommand.h"
#include "tundramodel.h"
#include "tundracommandvisitor.h"

//! Block Command
/*!
	TundraBlockCommand is used for any command that needs to operate
	on a "block" of the canvas. For example, Flip X and Flip Y. The block
	command uses a block strategy, passed in via the constructor, to operate
	on the specified portion of the canvas.
	\sa TundraBlockStrategy
*/
class TundraBlockCommand :
            public TundraCommand
{
public:
    //! Constructor
    /*!
    	\param m A pointer to the model
    	\param tl The top left point for the block
    	\param br The bottom right point for the block
    	\param s A pointer to the block strategy to use
    */
    TundraBlockCommand(TundraModel* m, QPoint& tl, QPoint& br, TundraBlockStrategy* s);
    //! Destructor
    ~TundraBlockCommand(void);
    //! Execute
    /*!
    	This is called by the TundraCommandProcessor to execute the block command
    */
    void execute();
    //! Unexecute
    /*!
    	This is called by the TundraCommandProcessor to undo the block command
    */
    void unexecute();
    //! Accept visitor
    /*!
    	We use the visitor pattern to allow commands to be easily run over network connections
    	\param v A pointer to the command visitor
    */
    void accept(TundraCommandVisitor* v) { v->visitBlockCommand(this); }
    //! Specify that this is a remote (network) command
    void setRemote() { is_remote = true; }
    TundraBlockStrategy* strategy;
    QPoint top_left;
    QPoint bottom_right;
private:
    TundraModel* model;
    TundraDoc** doc_block;
    int width;
    int height;
    bool is_remote;
};

#endif
