/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRABLOCKSTRATEGY_H
#define TUNDRABLOCKSTRATEGY_H

#include <qpoint.h>

#include "tundramodel.h"

class TundraBlockStrategyVisitor;

//! Block Strategy Interface
/*!
	This is an interface for a block strategy. Block strategies are used by
	block commands to operate on a block of the canvas.
	\sa TundraBlockCommand
*/
class TundraBlockStrategy
{
public:
    //! Execute
    /*!
    	This is called by the TundraBlockCommand using this strategy to operate on the specified block
    	\param m A pointer to the model
    	\param tl The point at the top left of the block
    	\param br The point at the bottom right of the block
    */
    virtual void execute(TundraModel* m, QPoint& tl, QPoint& br)=0;

    //! Accept visitor
    /*!
    	We use the visitor pattern to run commands over the network easily.
    */
    virtual void accept(TundraBlockStrategyVisitor* s)=0;
};

#endif
