/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRABLOCKSTRATEGYVISITOR_H
#define TUNDRABLOCKSTRATEGYVISITOR_H

class TundraEraseStrategy;
class TundraFillAttributeStrategy;
class TundraFillBgStrategy;
class TundraFillBothStrategy;
class TundraFillCharStrategy;
class TundraFillFgStrategy;
class TundraPasteStrategy;
class TundraFlipXStrategy;
class TundraFlipYStrategy;

//! Interface for a block strategy visitor

/*! 
	The block strategy visitor is primarily used by the network code to generate
	full block commands, but may be used for anything that needs to visit a list
	of block strategies and perform an action accordingly.
	
	\sa TundraNetBlockStrategyVisitor
*/
class TundraBlockStrategyVisitor
{
public:
    virtual void visitEraseStrategy(TundraEraseStrategy* s)=0;
    virtual void visitFillAttributeStrategy(TundraFillAttributeStrategy* s)=0;
    virtual void visitFillBgStrategy(TundraFillBgStrategy* s)=0;
    virtual void visitFillBothStrategy(TundraFillBothStrategy* s)=0;
    virtual void visitFillCharStrategy(TundraFillCharStrategy* s)=0;
    virtual void visitFillFgStrategy(TundraFillFgStrategy* s)=0;
    virtual void visitPasteStrategy(TundraPasteStrategy* s)=0;
    virtual void visitFlipXStrategy(TundraFlipXStrategy* s)=0;
    virtual void visitFlipYStrategy(TundraFlipYStrategy* s)=0;
};

#endif
