/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRABYTEBUFFER_H
#define TUNDRABYTEBUFFER_H

#include "qobject.h"

//! A relatively simple byte buffer
/*! 
	The byte buffer is used primarily in the network code, for buffering partial commands that
	were received, either when we've only received part of a command, or if we receive more than
	one full command. 
*/
class TundraByteBuffer
{
public:
	//! Constructor
    TundraByteBuffer(void);
	
	//! Destructor
    ~TundraByteBuffer(void);

	//! Add to the byte buffer
	/*!
		Call this to data to the existing byte buffer
		\param d A pointer to an array of characters to store
		\param l The number of characters in the array pointed to by d
	*/
    void add(char* d, Q_ULONG l) {
        Q_ULONG new_len = len + l;
        char* new_data = new char[new_len];

        for(Q_ULONG i = 0; i < len; i++)
            new_data[i] = data[i];

        for(Q_ULONG i = len; i < new_len; i++)
            new_data[i] = d[i-len];

        delete [] data;

        data = new_data;
        len = new_len;
    }

	//! Add to the current byte buffer, return sum, and clear
	/*!
		Call this to add data to the byte buffer, then clear the internal
		buffer and return the entire new buffer.
		
		\param d A pointer to an array of characters to add to the existing buffer
		\param l The number of characters in the array pointed to by d
		\param new_len The length of the new array will be stored in this reference
	*/
    char* addAndClear(char* d, Q_ULONG l, Q_ULONG& new_len) {
        new_len = len + l;
        char* new_data = new char[new_len];

        for(Q_ULONG i = 0; i < len; i++)
            new_data[i] = data[i];

        for(Q_ULONG i = len; i < new_len; i++)
            new_data[i] = d[i-len];

        delete [] data;
        data = NULL;

        len = 0;

        return new_data;
    }

	//! Get the length of the buffer
	/*!
		This returns the length of the current buffer
	*/
    Q_ULONG length() const {
        return len;
    }

private:
    Q_ULONG len;
    char* data;
};

#endif
