/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundracanvasgroup.h"

TundraCanvasGroup::TundraCanvasGroup(TundraModel* model, TundraCommandProcessor* command_processor, int width, int height, QWidget* parent, const char* name, WFlags f)
        : QHBox(parent, name, f)
{
    canvas = new TundraCanvas(model, command_processor, width, this, "Tundra Canvas");
    sb = new QScrollBar(0, NUM_ROWS-25, 1, 25, 0, QScrollBar::Vertical, this, "Canvas Scrollbar");

    canvas->resize(width,height);
    sb->resize(15,height);
    sb->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::MinimumExpanding));

    canvas->setFocus();

    QObject::connect(model, SIGNAL(modified()), this, SLOT(modelModified()));
    QObject::connect(sb, SIGNAL(valueChanged(int)), this, SLOT(scrollbarMoved(int)));
    QObject::connect(canvas, SIGNAL(topLineChanged()), this, SLOT(canvasTopLineChanged()));
}


TundraCanvasGroup::~TundraCanvasGroup()
{
}

void TundraCanvasGroup::modelModified() {
    canvas->repaint(false);
}

void TundraCanvasGroup::scrollbarMoved(int value) {
    if(value != canvas->getTopLine()) {
        canvas->setTopLine(value);
    }
}

void TundraCanvasGroup::canvasTopLineChanged() {
    canvas->repaint(false);

    if(canvas->getTopLine() != sb->value()) {
        sb->setValue(canvas->getTopLine());
    }
}

void TundraCanvasGroup::resetCanvas() {
    canvas->setTopLine(0);
    sb->setValue(canvas->getTopLine());
}

void TundraCanvasGroup::resizeCanvas(int width) {
    canvas->setWidth(width);
}

void TundraCanvasGroup::forwardResize() {
    sb->setMaxValue(NUM_ROWS-canvas->height()/(canvas->width()/40));
    sb->setPageStep(canvas->height()/(canvas->width()/40));
    canvasTopLineChanged();
}

void TundraCanvasGroup::sendRepaint(bool erase) {
    canvas->repaint(erase);
}
