/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACANVASGROUP_H
#define TUNDRACANVASGROUP_H

#include <qhbox.h>
#include <qscrollbar.h>

#include "tundracanvas.h"
#include "tundramodel.h"

//! Container for a canvas and associated scrollbar
/*! 
	The canvas group is a container widget that holds the canvas
	and a scrollbar, and is responsible for mainting the relationship
	between the two. This is an application of the Mediator design
	pattern.
*/
class TundraCanvasGroup : public QHBox
{
    Q_OBJECT
public:
	//! Constructor
	/*! 
		\param model A pointer to the model
		\param command_processor A pointer to the main command processor
		\param width The width of the canvas
		\param height The height of the canvas
	*/
    TundraCanvasGroup(TundraModel* model, TundraCommandProcessor* command_processor, int width, int height, QWidget* parent=0, const char* name=0, WFlags f=0);
    
	//! Destructor
	~TundraCanvasGroup();
    
	//! Scrolls the canvas back up to its top line
	void resetCanvas();

	//! Adjust scrollbar for current canvas size
    void forwardResize();

	//! Force the canvas to repaint
	/*!
		\param erase TRUE to erase the background of the canvas, FALSE otherwise
	*/
    void sendRepaint(bool erase);

	//! Resize the canvas to a new width
	/*!
		\param width The new width of the canvas
	*/
    void resizeCanvas(int width);

	//! Get a pointer to the canvas widget
	/*!
		\return A pointer to the canvas widget
	*/
    TundraCanvas* getCanvas() const { return canvas; }
public slots:
	//! The model was modified, so we need to repaint the canvas
    void modelModified();

	//! Adjust the top line of the canvas
	/*!
		\param value The new scrollbar value
	*/
    void scrollbarMoved(int value);

	//! Repaint the canvas and adjust the scrollbar to the new top line
    void canvasTopLineChanged();
private:
    TundraCanvas* canvas;
    QScrollBar* sb;

};

#endif
