/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACHARACTERCOMMAND_H
#define TUNDRACHARACTERCOMMAND_H

#include "tundracommand.h"
#include "tundramodel.h"
#include "tundracommandvisitor.h"


//! Place a single block on the canvas
/*!
	The character command is used to place a single block on the canvas when insert mode is off. 
	It directly alters the model.

	\sa TundraInsertCommand
*/
class TundraCharacterCommand :
            public TundraCommand
{
public:
	//! Constructor
	/*!
		\param m A pointer to the model
		\param doc The new block to use
		\param x The x location on which to place the block
		\param y The y location on which to place the block
	*/
    TundraCharacterCommand(TundraModel* m, TundraDoc& doc, int x, int y);
    
	//! Destructor
	~TundraCharacterCommand(void);

	//! Place the new block on the canvas (in the model)
    void execute();

	//! Replace the new block with the original block
    void unexecute();

	//! Set this for a backspace press
	/*!
		We set this for a backspace press so when know whether to advance the cursor 
		or reverse it
	*/
    void setBackspace() { is_backspace = true; }

	//! Accept a visitor
    void accept(TundraCommandVisitor* v) { v->visitCharacterCommand(this); }
    
	//! Set this for commands coming in over the network
	/*!
		We set this for network commands so we do not move the cursor
	*/
	void setRemote() { is_remote = true; }

	//! The new block
    TundraDoc new_doc;

	//! The x location for the new block
    int my_x;

	//! The y location for the new block
    int my_y;

private:
    TundraDoc old_doc;
    TundraModel* model;
    bool is_backspace;
    bool is_remote;

};

#endif
