/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACHARPREVIEWWIDGETIMAGE_H
#define TUNDRACHARPREVIEWWIDGETIMAGE_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qpainter.h>

#define WIDTH 9
#define HEIGHT 16

#include "tundramodel.h"

//! A custom widget for rendering a single character
/*!
	This is used to render a single character in both the character set preview
	and the fill character dialogs.
*/
class TundraCharPreviewWidgetImage : public QWidget
{
    Q_OBJECT
public:
	//! Constructor
	/*!
		\param m A pointer to the model
		\param fkey The F-key to render in the image
		\param invertPixels Whether to render as black on white or white on black
	*/
    TundraCharPreviewWidgetImage(TundraModel* m, int fkey, bool invertPixels = true, QWidget *parent = 0, const char *name = 0);

	//! Destructor
    ~TundraCharPreviewWidgetImage();

	//! Standard Qt paintEvent override
	/*!
		We override paintEvent so we can do custom drawing of the widget in order to render
		the character
	*/
    void paintEvent(QPaintEvent*);

	//! Standard qt sizeHint override
    QSize sizeHint() const { return QSize(WIDTH, HEIGHT); }

	//! The current key to render
    int this_key;

private:
    TundraModel* model;
    bool invert;

public slots:
    void renderKey(int new_key);
};

#endif
