/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrachatwindow.h"

TundraChatWindow::TundraChatWindow(Orientation o, QWidget *parent, const char *name)
        : QSplitter(o, parent, name), nick("me")
{
    chat_window = new QTextEdit(this);
    text_entry = new QTextEdit(this);
    chat_window->setReadOnly(true);
    chat_window->resize(500,110);
    text_entry->resize(500,30);
    text_entry->setMinimumSize(QSize(500,20));
    resize(500,140);

    QObject::connect(text_entry, SIGNAL(returnPressed()), this, SLOT(enterKeyPressed()));
}


TundraChatWindow::~TundraChatWindow()
{
}

void TundraChatWindow::showUserLogin(QString name) {
    QString new_line;
    if(chat_window->text().length() > 0)
        new_line = QString("\n");

    QScrollBar* sb = chat_window->verticalScrollBar();

    bool update = true;
    if(sb->value() != sb->maxValue())
        update = false;

    chat_window->setText(chat_window->text()+new_line+(QString("*** ")+name+QString(" logged in")));
    chat_window->sync();

    if(update)
        sb->setValue(sb->maxValue());
}

void TundraChatWindow::showUserMessage(QString name, QString message) {
    QTime current_time = QTime::currentTime(LocalTime);

    QScrollBar* sb = chat_window->verticalScrollBar();
    bool update = true;
    if(sb->value() != sb->maxValue())
        update = false;

    chat_window->setText(chat_window->text()+QString("\n")+current_time.toString("[hh:mm] ")+QString("<")+name+QString("> ")+message);

    chat_window->sync();

    if(update)
        sb->setValue(sb->maxValue());
}

void TundraChatWindow::enterKeyPressed() {
    QString text = text_entry->text();
    text_entry->setText("");
    text.truncate(text.length()-1);
    showUserMessage(nick, text);
    emit messageWritten(text);
}

void TundraChatWindow::showDisconnected() {
    chat_window->setText(chat_window->text()+QString("\n*** Disconnected"));

    chat_window->sync();

    QScrollBar* sb = chat_window->verticalScrollBar();
    sb->setValue(sb->maxValue());
}

void TundraChatWindow::showUserPart(QString name) {

    QScrollBar* sb = chat_window->verticalScrollBar();
    bool update = true;
    if(sb->value() != sb->maxValue())
        update = false;

    chat_window->setText(chat_window->text()+QString("\n*** ")+name+QString(" left"));

    chat_window->sync();

    if(update)
        sb->setValue(sb->maxValue());
}
