/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACHATWINDOW_H
#define TUNDRACHATWINDOW_H

#include <qtextedit.h>
#include <qsplitter.h>
#include <qdatetime.h>

//! The chat window (chat buffer and text entry)
/*! 
	The chat window contains both an autoscrolling buffer of messages
	and a text edit box for entering messages. It subclasses QSplitter
	so that the user can resize the two widgets in relationship to
	each other
*/
class TundraChatWindow : public QSplitter
{
    Q_OBJECT
public:
	//! Constructor
    TundraChatWindow(Orientation o, QWidget *parent = 0, const char *name = 0);
	
	//! Destructor
    ~TundraChatWindow();
	
	//! Standard Qt sizeHint override
    QSize sizeHint() const { return QSize(500,140); }
	
	//! Sets the user's nickname
    void setNick(QString n) { nick = n; }

public slots:
	//! Print a message showing a new user login
	/*!
		\param name The name of the new user
	*/
    void showUserLogin(QString name);
	
	//! Display a message from a user in the chat buffer
	/*!
		\param name The name of the user from which the message came
		\param message The message to display
	*/
    void showUserMessage(QString name, QString message);

	//!	Send a message when the enter key is pressed
    void enterKeyPressed();

	//! Print a message showing that the user was disconnected from the server
    void showDisconnected();

	//! Print a message showing that a user left
    void showUserPart(QString name);
signals:
	//! Notify that the user wrote a message and hit enter
    void messageWritten(QString);
private:
    QTextEdit* chat_window;
    QTextEdit* text_entry;
    QString nick;

};

#endif
