/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundraclientsocket.h"

TundraClientSocket::TundraClientSocket(TundraChatWindow* cw, TundraUserList* ul, TundraModel* m, QString n, QObject* parent, const char* name) : QSocket(parent, name), nick(n), logged_in(false), model(m), chat_window(cw), user_list(ul)
{
}

TundraClientSocket::~TundraClientSocket(void)
{
    QMap<int, TundraNetCommandProcessor*>::iterator processor_it;
    for(processor_it = processor_map.begin(); processor_it != processor_map.end(); processor_it++) {
        if((*processor_it) != NULL) {
            delete (*processor_it);
        }
    }

    processor_map.clear();

    QMap<int, TundraByteBuffer*>::iterator byte_buffer_it;
    for(byte_buffer_it = byte_buffer_map.begin(); byte_buffer_it != byte_buffer_map.end(); byte_buffer_it++) {
        if((*byte_buffer_it) != NULL) {
            delete (*byte_buffer_it);
        }
    }

    byte_buffer_map.clear();


}

void TundraClientSocket::sendColorMode() {
    char* command = new char[2];
    command[0] = 13;
    command[1] = (char)model->getColorMode();

    writeBlock(command, 2);

    delete [] command;
}

void TundraClientSocket::connectToServer(QString ip, Q_UINT16 port) {
    QObject::connect(this, SIGNAL(connected()), this, SLOT(connectionMade()));
    connectToHost(ip, port);
}

void TundraClientSocket::connectionMade() {

    bool login_failed = false;

    // Attempt to login
    char* login_command = new char[nick.length()+1];
    login_command[0] = (char)nick.length();
    for(uint i = 0; i < nick.length(); i++) {
        QCharRef current_ref = nick.at(i);
        QChar current_char = current_ref;
        login_command[i+1] = current_char;
    }

    if(writeBlock(login_command, nick.length()+1) != -1) {
        // We sent the login command
        logged_in = true;
        processor_map[0] = new TundraNetCommandProcessor(model);
        byte_buffer_map[0] = new TundraByteBuffer();
        QObject::connect(this, SIGNAL(readyRead()), this, SLOT(readSocket()));
        QObject::connect(processor_map[0], SIGNAL(userLoggedIn(QString)), user_list, SLOT(addUser(QString)));
        QObject::connect(processor_map[0], SIGNAL(userLoggedIn(QString)), chat_window, SLOT(showUserLogin(QString)));
        QObject::connect(processor_map[0], SIGNAL(userLoggedOut(QString)), chat_window,
                         SLOT(showUserPart(QString)));
        QObject::connect(processor_map[0], SIGNAL(userLoggedOut(QString)), user_list, SLOT(removeUser(QString)));
        QObject::connect(processor_map[0], SIGNAL(userMessage(QString,QString)), chat_window, SLOT(showUserMessage(QString,QString)));
    }
    else
        login_failed = true;
}

void TundraClientSocket::readSocket() {
    int command_length = bytesAvailable();
    char* data = new char[command_length];

    readBlock(data, command_length);

    Q_ULONG new_length;

    // Add the old byte buffer to the command
    data = byte_buffer_map[0]->addAndClear(data, command_length, new_length);
    command_length = new_length;

    int processor = (int)data[0];

    if(processor >= 0) {
        // data[0] is the processor map number to use
        if(!processor_map.contains(processor)) {
            processor_map[processor] = new TundraNetCommandProcessor(model);
            QObject::connect(processor_map[processor], SIGNAL(userMessage(QString,QString)), chat_window, SLOT(showUserMessage(QString,QString)));
        }

        int bytes_processed;
        while((bytes_processed = processor_map[processor]->parseNetCommand(data+1, command_length)) != 0) {
            data += (bytes_processed+1);
            command_length -= (bytes_processed+1);

            if(command_length == 0)
                break;
        }

        // Buffer any unprocessed data
        if(command_length > 0) {
            byte_buffer_map[0]->add(data, command_length);
        }
    }
    else {
        QMessageBox::critical(NULL, "Bad Message", "ERROR: Received bad message from server");
    }
}

void TundraClientSocket::disconnectFromServer() {
    close();
}

void TundraClientSocket::sendAnsi() {
    int character_count = 0;

    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            TundraDoc* doc = model->getItem(j, i);
            if(doc->character != 0)
                character_count++;
        }
    }

    char* command = new char[((character_count*17)+7)];

    command[0] = 0;
    unpackInteger(character_count, command+1);
    command[5] = '|';
    command[6] = (char)model->getColorMode();

    int count = 7;

    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            TundraDoc* doc = model->getItem(j, i);
            if(doc != NULL) {
                if(doc->character != 0) {
                    unpackInteger(j, command+count);
                    count += 4;
                    unpackInteger(i, command+count);
                    count += 4;
                    command[count] = doc->character;
                    count++;
                    unpackInteger(doc->bg_color, command+count);
                    count += 4;
                    unpackInteger(doc->fg_color, command+count);
                    count += 4;
                }
            }
            else {
                // Something went wrong in getting an item to send
            }
        }
    }

    writeBlock(command, count);
    delete [] command;
}

void TundraClientSocket::sendTextMessage(QString message) {
    int command_length = nick.length()+5+message.length()+4;
    char* command = new char[command_length];
    command[0] = 8;
    unpackInteger(nick.length(), command+1);
    for(unsigned int i = 0; i < nick.length(); i++) {
        QChar ch = nick.at(i);
        command[i+5] = (char)ch;
    }
    unpackInteger(message.length(), command+5+nick.length());
    for(unsigned int i = 0; i < message.length(); i++) {
        QChar ch = message.at(i);
        command[i+5+nick.length()+4] = (char)ch;
    }

    writeBlock(command, command_length);
}
