/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACLIENTSOCKET_H
#define TUNDRACLIENTSOCKET_H

#include <qmap.h>
#include <qsocket.h>
#include <qmessagebox.h>

#include "tundrachatwindow.h"
#include "tundramodel.h"
#include "tundranetcommandprocessor.h"
#include "tundrabytebuffer.h"
#include "tundrauserlist.h"
#include "utils.h"

//! Client socket for connecting to a server
/*! 
	The client socket is responsible for connecting to a server and
	reading and writing commands over the network.
*/
class TundraClientSocket :
            public QSocket
{
    Q_OBJECT
public:
	//! Constructor
	/*!
		\param cw A pointer to the chat window
		\param ul A pointer to the user list
		\param m A pointer to the model
		\param n The nick of the user
	*/
    TundraClientSocket(TundraChatWindow* cw, TundraUserList* ul, TundraModel* m, QString n, QObject * parent = 0, const char * name = 0);
    
	//! Destructor
	~TundraClientSocket(void);

	//! Connect to a server
    void connectToServer(QString ip, Q_UINT16 port);

	//! Return if we are logged in or not
    bool isLoggedIn() { return logged_in; }

	//! Disconnect from the server
    void disconnectFromServer();

	//! Return a pointer to the server's command processor
    TundraNetCommandProcessor* getNetCommandProcessor() const { return processor_map[0]; }
    
	//! Send the current drawing to the server
	void sendAnsi();

	//! Send the current color mode to the server
    void sendColorMode();
public slots:
	//! Attempt to log into the server
    void connectionMade();

	//! Read data from the socket
    void readSocket();

	//! Send a text message to the server
    void sendTextMessage(QString message);
private:
    QString nick;
    bool logged_in;
    QMap<int, TundraNetCommandProcessor*> processor_map;
    QMap<int, TundraByteBuffer*> byte_buffer_map;
    TundraModel* model;
    TundraChatWindow* chat_window;
    TundraUserList* user_list;
};

#endif
