/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundracolorbox.h"

TundraColorBox::TundraColorBox(TundraModel* m, QColor color, int index, bool bgok, QWidget* parent, const char* name) : QFrame(parent, name),  my_color(color), color_index(index), bg_okay(bgok), model(m) {
    setFrameShape(QFrame::WinPanel);
    setFrameShadow(QFrame::Sunken);
    setFixedSize(15,15);
    setBackgroundColor(color);
    setAcceptDrops(TRUE);
}

TundraColorBox::~TundraColorBox() {

}

void TundraColorBox::mouseReleaseEvent(QMouseEvent* e) {
    if(e->button() == LeftButton) {
        // change foreground
        if(model->getColorMode() == COLOR_4BIT) {
            model->setCurrentFg(color_index);
            e->accept();
        }
        else if(model->getColorMode() == COLOR_24BIT) {
            int packed_color = ((my_color.red() & 0x000000FF) << 16) | ((my_color.green() & 0x000000FF) << 8) | (my_color.blue() & 0x000000FF);

            model->setCurrentFg(packed_color);
            e->accept();
        }
    }
    else if(e->button() == RightButton) {
        // change background if bg_okay
        if(model->getColorMode() == COLOR_4BIT) {
            if(bg_okay) {
                model->setCurrentBg(color_index);
                e->accept();
            }
        }
        else if(model->getColorMode() == COLOR_24BIT) {
            int packed_color = ((my_color.red() & 0x000000FF) << 16) | ((my_color.green() & 0x000000FF) << 8) | (my_color.blue() & 0x000000FF);

            model->setCurrentBg(packed_color);
            e->accept();
        }
    }

}

void TundraColorBox::mouseDoubleClickEvent(QMouseEvent* e) {
    if(e->button() == LeftButton) {
        if(model->getColorMode() == COLOR_24BIT) {
            QColor color;

            color = QColorDialog::getColor(my_color);

            if(color.isValid()) {
                int packed_color = packColor(color.red(), color.green(), color.blue());

                color_index = packed_color;
                my_color = color;
                setBackgroundColor(my_color);

                model->setCurrentFg(packed_color);

                e->accept();
            }
        }
    }
}

void TundraColorBox::mouseMoveEvent(QMouseEvent* e) {
    if(model->getColorMode() == COLOR_24BIT) {
        QDragObject* d = new QColorDrag(my_color, this);
        d->dragCopy();
        e->accept();
    }
}

void TundraColorBox::contextMenuEvent(QContextMenuEvent* e) {
    e->accept();
}

void TundraColorBox::dragEnterEvent(QDragEnterEvent* e) {
    e->accept(QColorDrag::canDecode(e));
}

void TundraColorBox::dropEvent(QDropEvent* e) {
    QColor color;

    if(QColorDrag::decode(e, color)) {
        int packed_color = packColor(color.red(), color.green(), color.blue());

        color_index = packed_color;
        my_color = color;
        setBackgroundColor(my_color);

        model->setCurrentFg(packed_color);
    }
}
