/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACOLORBOX_H
#define TUNDRACOLORBOX_H

#include <qcolordialog.h>
#include <qdragobject.h>
#include <qframe.h>

#include "tundramodel.h"

//! An individual swatch in the palette widget
/*!
	The TundraColorBox creates a clickable, drag and droppable color box
*/
class TundraColorBox : public QFrame {

public:
	//! Constructor
	/*! 
		\param m A pointer to the model
		\param color The initial background color of the swatch
		\param index The color index associated with this swatch
		\param bgok Whether or not this color is a valid bg color
	*/
    TundraColorBox(TundraModel* m, QColor color, int index, bool bgok, QWidget* parent, const char* name=0);
    
	//! Destructor
	~TundraColorBox();

	//! Standard Qt mouseMoveEvent override
	/*!	
		We override mouseMoveEvent so that we can create a dragEvent to allow
		dragging and dropping between color boxes
	*/
    void mouseMoveEvent(QMouseEvent*);

	//! Standard Qt mouseReleaseEvent override
	/*!
		We override mouseReleaseEvent to set the foreground or background color
		after to the color in this box after a mouse press
	*/
    void mouseReleaseEvent(QMouseEvent* e);

	//! Standard Qt mouseDoubleClickEvent override
	/*!
		We override mouseDoubleClickEvent to bring up the color picker when
		we are in 24-bit mode
	*/
    void mouseDoubleClickEvent(QMouseEvent* e);

	//! Disable Qt's toolbar context menu so that we can receive a right-click
    void contextMenuEvent(QContextMenuEvent* e);

	//! Determine if we can decode this particular drag
    void dragEnterEvent(QDragEnterEvent* e);

	//! Handle the drop from a drag and drop
    void dropEvent(QDropEvent* e);

private:
    QColor my_color;
    int color_index;
    bool bg_okay;
    TundraModel* model;
};

#endif
