/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundracommandprocessor.h"
#include "tundranetcommandvisitor.h"

TundraCommandProcessor::TundraCommandProcessor(TundraModel* m, TundraNetCommandVisitor* v)
{
    model = m;
    command_pos = -1;
    net_command_visitor = v;
}

TundraCommandProcessor::~TundraCommandProcessor(void)
{
}

TundraCharacterCommand* TundraCommandProcessor::newCharacterCommand(int x, int y, unsigned char character,
        int bg_color, int fg_color) {
    TundraDoc doc;
    doc.character = character;
    doc.bg_color = bg_color;
    doc.fg_color = fg_color;

    return new TundraCharacterCommand(model, doc, x, y);
}

TundraDeleteCommand* TundraCommandProcessor::newDeleteCommand(int x, int y) {
    return new TundraDeleteCommand(model, x, y);
}

TundraBlockCommand* TundraCommandProcessor::newBlockCommand(QPoint& tl, QPoint& br, TundraBlockStrategy* s) {
    return new TundraBlockCommand(model, tl, br, s);
}

TundraMoveCommand* TundraCommandProcessor::newMoveCommand(QPoint& tl, QPoint& br) {
    return new TundraMoveCommand(model, tl, br);
}

TundraInsertCommand* TundraCommandProcessor::newInsertCommand(int x, int y, unsigned char character, int bg_color, int fg_color) {
    TundraDoc doc;
    doc.character = character;
    doc.bg_color = bg_color;
    doc.fg_color = fg_color;

    return new TundraInsertCommand(model, doc, x, y);
}

void TundraCommandProcessor::process(TundraCommand* command) {
    if(net_command_visitor != NULL)
        command->accept(net_command_visitor);

    command->execute();

    // Remove any steps that have already been undone
    int new_pos = command_pos+1;
    while(!command_list.empty() && (new_pos < (signed)command_list.size())) {
        delete command_list[new_pos];
        new_pos++;
    }

    command_list.resize(command_pos+1);

    command_list.push_back(command);
    command_pos = command_list.size()-1;

    emit canUndo();
    emit cantRedo();
}

void TundraCommandProcessor::undo() {
    // Unexecute the current command and move back in the list
    if(!command_list.empty() && command_pos > -1) {
        if(net_command_visitor != NULL)
            net_command_visitor->undo();

        command_list[command_pos]->unexecute();
        command_pos--;

        // Check if we can no longer undo
        if(command_pos == -1)
            emit cantUndo();

        emit canRedo();
    }
}

void TundraCommandProcessor::redo() {
    // Execute the next command and move forward
    if(!command_list.empty() && command_pos+1 < (signed)command_list.size()) {
        if(net_command_visitor != NULL)
            net_command_visitor->redo();

        command_list[++command_pos]->execute();

        if(command_pos+1 == (int)command_list.size())
            emit cantRedo();

        emit canUndo();
    }
}

void TundraCommandProcessor::reset() {
    for(unsigned int i = 0; i < command_list.size(); i++) {
        if(command_list[i] != NULL) {
            delete command_list[i];
            command_list[i] = NULL;
        }
    }

    command_list.clear();
    command_pos = -1;
    emit cantUndo();
    emit cantRedo();
}
