/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRACOMMANDPROCESSOR_H
#define TUNDRACOMMANDPROCESSOR_H

#include "tundramodel.h"
#include "tundrablockcommand.h"
#include "tundracharactercommand.h"
#include "tundradeletecommand.h"
#include "tundrainsertcommand.h"
#include "tundramovecommand.h"
#include "tundraerasestrategy.h"
#include "tundrafillattributestrategy.h"
#include "tundrafillfgstrategy.h"
#include "tundrafillbgstrategy.h"
#include "tundrafillbothstrategy.h"
#include "tundrafillcharstrategy.h"
#include "tundraflipxstrategy.h"
#include "tundraflipystrategy.h"
#include "tundrapastestrategy.h"

class TundraNetCommandVisitor;

#include <qvaluevector.h>

//! Command processor supporting undo/redo
/*!
	The command processor is responsible for maintaining a command history, thus 
	executing and unexecuting commands. 
*/
class TundraCommandProcessor : public QObject
{
    Q_OBJECT
public:
	//! Constructor
	/*!
		\param m A pointer to the model
		\param v A pointer to an optional net command visitor
	*/
    TundraCommandProcessor(TundraModel* m, TundraNetCommandVisitor* v=NULL);

	//! Destructor
    ~TundraCommandProcessor(void);

	//! Create a TundraCharacterCommand
	/*!
		\param x The x location of the new character
		\param y The y location of the new character
		\param character The value of the new character
		\param bg_color The background color of the new character
		\param fg_color The foreground color of the new character
	*/
    TundraCharacterCommand* newCharacterCommand(int x, int y, unsigned char character,
            int bg_color, int fg_color);

	//! Create a TundraDeleteCommand
	/*!
		\param x The x location of the new character
		\param y The y location of the new character
	*/
    TundraDeleteCommand* newDeleteCommand(int x, int y);

	//! Create a TundraInsertCommand
	/*!
		\param x The x location of the new character
		\param y The y location of the new character
		\param character The value of the new character
		\param bg_color The background color of the new character
		\param fg_color The foreground color of the new character
	*/
    TundraInsertCommand* newInsertCommand(int x, int y, unsigned char character,
                                          int bg_color, int fg_color);

	//! Create a TundraBlockCommand
	/*!
		\param tl A QPoint representing the top left corner of the block
		\param br A QPoint representing the bottom right corner of the block
		\param s A pointer to the block strategy to use in this command
	*/
    TundraBlockCommand* newBlockCommand(QPoint& tl, QPoint& br, TundraBlockStrategy* s);

	//! Create a TundraMoveCommand
	/*!
		\param tl A QPoint representing the top left corner of the block
		\param br A QPoint representing the bottom right corner of the block
	*/
    TundraMoveCommand* newMoveCommand(QPoint& tl, QPoint& br);
    
	//! Process a command
	/*!
		\param command A pointer to the command to process
	*/
	void process(TundraCommand* command);

	//! Undo the last command in the command list
    void undo();

	//! Redo the next command in the command list
    void redo();

	//! Delete the command history
    void reset();
protected:
	//! A pointer to the model
    TundraModel* model;
private:
    QValueVector<TundraCommand*> command_list;
    int command_pos;
    TundraNetCommandVisitor* net_command_visitor;
signals:
	//! Notify that an undo is possible
    void canUndo();

	//! Notify that an undo is not possible
    void cantUndo();

	//! Notify that a redo is possible
    void canRedo();

	//! Notify that a redo is not possible
    void cantRedo();
};

#endif
