/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundraconnectdlg.h"

#include <qvariant.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/*
 *  Constructs a TundraConnectDlg as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
TundraConnectDlg::TundraConnectDlg( QWidget* parent, const char* name, bool modal, WFlags fl )
        : QDialog( parent, name, modal, fl )
{
    if ( !name )
        setName( "TundraConnectDlg" );

    ip_label = new QLabel( this, "ip_label" );
    ip_label->setGeometry( QRect( 20, 20, 111, 21 ) );

    ip_edit = new QLineEdit( this, "ip_edit" );
    ip_edit->setGeometry( QRect( 20, 40, 191, 21 ) );
    ip_edit->setMaxLength(256);

    port_label = new QLabel( this, "port_label" );
    port_label->setGeometry( QRect( 20, 70, 81, 21 ) );

    port_edit = new QLineEdit( this, "port_edit" );
    port_edit->setGeometry( QRect( 20, 90, 71, 21 ) );
    port_edit->setMaxLength(5);

    nick_label = new QLabel( this, "nick_label" );
    nick_label->setGeometry( QRect( 20, 120, 91, 21 ) );

    nick_edit = new QLineEdit( this, "nick_edit" );
    nick_edit->setGeometry( QRect( 20, 140, 141, 20 ) );
    nick_edit->setMaxLength(32);

    connect_button = new QPushButton( this, "connect_button" );
    connect_button->setGeometry( QRect( 290, 30, 80, 31 ) );

    cancel_button = new QPushButton( this, "cancel_button" );
    cancel_button->setGeometry( QRect( 290, 70, 81, 31 ) );
    languageChange();
    resize( QSize(398, 193).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    QObject::connect(connect_button, SIGNAL(released()), this, SLOT(checkFields()));
    QObject::connect(cancel_button, SIGNAL(released()), this, SLOT(reject()));
}

/*
 *  Destroys the object and frees any allocated resources
 */
TundraConnectDlg::~TundraConnectDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void TundraConnectDlg::languageChange()
{
    setCaption( tr( "Connect to Server" ) );
    ip_label->setText( tr( "IP Address:" ) );
    port_label->setText( tr( "Port:" ) );
    nick_label->setText( tr( "Nick:" ) );
    connect_button->setText( tr( "Connect" ) );
    cancel_button->setText( tr( "Cancel" ) );
}

void TundraConnectDlg::checkFields() {
    bool error = false;

    if(ip_edit->text().length() < 1) {
        QMessageBox::critical(this, "Error", "You must specify a valid server  address");
        error = true;
    }
    if(nick_edit->text().length() < 1) {
        QMessageBox::critical(this, "Error", "You must specify a nick");
        error = true;
    }
    if((port_edit->text().toInt() > 65535) || (port_edit->text().toInt() < 1)) {
        QMessageBox::critical(this, "Error", "Port number must be between 1 and 65535");
        error = true;
    }
    if(!error) {
        accept();
    }
}

