/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundradrawwindow.h"
#include "smallicon.h"
#include "fileopen.xpm"
#include "filenew.xpm"
#include "filesave.xpm"
#include "filesaveas.xpm"
#include "exit.xpm"
#include "undo.xpm"
#include "redo.xpm"
#include "viewmag-.xpm"
#include "mode4.xpm"
#include "mode24.xpm"
#include "connect_creating.xpm"
#include "fileclose.xpm"
#include "eraser.xpm"
#include "fill.xpm"

#include <qaction.h>
#include <qmessagebox.h>
#include <qsizepolicy.h>
#include <qthread.h>

TundraDrawWindow::TundraDrawWindow(QString filename, QSplashScreen* s, QWidget* parent, const char* name, WFlags f)
        : QMainWindow( parent, name, f ), save_item(-1), splash(s)
{

    model = new TundraModel();
    client_socket = NULL;
    socket_server = NULL;

    setFocusProxy(0);
    setFocusPolicy(QWidget::StrongFocus);

    setIcon(QPixmap(small_icon_xpm));

    splash->message("Initializing command processor...", AlignBottom);
    net_command_visitor = new TundraNetCommandVisitor();
    command_processor = new TundraCommandProcessor(model, net_command_visitor);

    setCaption("TundraDraw - Untitled Document");
    setFocusPolicy(QWidget::StrongFocus);
    resize(660, 480);

    splash->message("Creating menus...", AlignBottom);

    main_menu = new QMenuBar(this);
    file_menu = new QPopupMenu();
    edit_menu = new QPopupMenu();
    view_menu = new QPopupMenu();
    tools_menu = new QPopupMenu();
    help_menu = new QPopupMenu();
    block_menu = new QPopupMenu();
    fill_menu = new QPopupMenu();
    mode_menu = new QPopupMenu();
    key_fill_menu = new QPopupMenu();

    mode_4bit_id = mode_menu->insertItem(QIconSet(QPixmap(mode4_xpm)), "4-bit Color", this, SLOT(colorMode4Bit()));
    mode_24bit_id = mode_menu->insertItem(QIconSet(QPixmap(mode24_xpm)), "24-bit Color", this, SLOT(colorMode24Bit()));

    mode_menu->setItemChecked(mode_4bit_id, true);

    fill_menu->insertItem("Attribute", this, SLOT(fillAttribute()));
    fill_menu->insertItem("Character", this, SLOT(fillCharacter()));
    fill_menu->insertItem("Both Character and Attribute", this, SLOT(fillCharacterAttribute()));
    fill_menu->insertItem("Foreground Color", this, SLOT(fillFgColor()));
    fill_menu->insertItem("Background Color", this, SLOT(fillBgColor()));

    block_menu->insertItem(QIconSet(QPixmap(fill_xpm)), "Fill", fill_menu);
    block_menu->insertItem(QIconSet(QPixmap(eraser_xpm)), "Erase\t\t\tE", this, SLOT(erase()));
    block_menu->insertItem("Flip X\t\t\tX", this, SLOT(flipX()));
    block_menu->insertItem("Flip Y\t\t\tY", this, SLOT(flipY()));

    file_menu->insertItem(QIconSet(QPixmap(filenew_xpm)), "New", this, SLOT(newDocument()), CTRL + Key_N);
    file_menu->insertItem(QIconSet(QPixmap(fileopen_xpm)), "Open...", this, SLOT(openDocument()), CTRL + Key_O);
    save_item = file_menu->insertItem(QIconSet(QPixmap(filesave_xpm)), "Save", this, SLOT(saveDocument()), CTRL + Key_S);
    file_menu->setItemEnabled(save_item, false);
    file_menu->insertItem(QIconSet(QPixmap(filesaveas_xpm)), "Save As...", this, SLOT(saveDocumentAs()));
#ifdef DEBUG
    file_menu->insertItem("Import 4-bit unit test data...", this, SLOT(importUnitTestData4()));
    file_menu->insertItem("Import 24-bit unit test data...", this, SLOT(importUnitTestData24()));
	file_menu->insertItem("Export unit test data...", this, SLOT(exportUnitTestData()));
#endif
    file_menu->insertSeparator();
    file_menu->insertItem(QIconSet(QPixmap(exit_xpm)), "Quit", this, SLOT(close()));

    key_fill_menu->insertItem("&Attribute", this, SLOT(fillAttribute()));
    key_fill_menu->insertItem("&Character", this, SLOT(fillCharacter()));
    key_fill_menu->insertItem("&Both Character and Attribute", this, SLOT(fillCharacterAttribute()));
    key_fill_menu->insertItem("&Foreground Color", this, SLOT(fillFgColor()));
    key_fill_menu->insertItem("Bac&kground Color", this, SLOT(fillBgColor()));


    undo_item_id = edit_menu->insertItem(QIconSet(QPixmap(undo_xpm)), "Undo", this, SLOT(undo()), CTRL + Key_Z);
    redo_item_id = edit_menu->insertItem(QIconSet(QPixmap(redo_xpm)), "Redo", this, SLOT(redo()), CTRL + Key_Y);
    edit_menu->insertSeparator();
    cut_clipboard_id = edit_menu->insertItem("Cut to clipboard", this, SLOT(cutToClipboard()), CTRL + Key_X);
    copy_clipboard_id = edit_menu->insertItem("Copy to clipboard", this, SLOT(copyToClipboard()), CTRL + Key_C);
    paste_clipboard_id = edit_menu->insertItem("Paste from clipboard", this, SLOT(pasteFromClipboard()), CTRL + Key_V);
    edit_menu->insertSeparator();
    copy_id = edit_menu->insertItem("Quick Copy\t\t\tC", this, SLOT(copy()));
    move_id = edit_menu->insertItem("Move\t\t\tM", this, SLOT(move()));
    edit_menu->insertSeparator();
    block_id = edit_menu->insertItem("Block", block_menu);
    edit_menu->insertItem("Mode", mode_menu);

    edit_menu->setItemEnabled(undo_item_id, FALSE);
    edit_menu->setItemEnabled(redo_item_id, FALSE);

    edit_menu->setItemEnabled(block_id, FALSE);
    edit_menu->setItemEnabled(copy_id, FALSE);
    edit_menu->setItemEnabled(move_id, FALSE);

    edit_menu->setItemEnabled(copy_clipboard_id, FALSE);
    edit_menu->setItemEnabled(cut_clipboard_id, FALSE);

    view_menu->setCheckable(true);
    preview_item_id = view_menu->insertItem(QIconSet(QPixmap(viewmag__xpm)), "Preview Window", this, SLOT(viewPreviewWindow()));
    emulate_dos_id = view_menu->insertItem("Emulate Dos Aspect", this, SLOT(emulateDosAspect()));

    start_server_id = tools_menu->insertItem("Start Server...", this, SLOT(startServer()));
    connect_to_server_id = tools_menu->insertItem(QIconSet(QPixmap(connect_creating_xpm)), "Connect to Server...", this, SLOT(connectToServer()));
    disconnect_from_server_id = tools_menu->insertItem(QIconSet(QPixmap(fileclose_xpm)), "Disconnect From Server...", this, SLOT(disconnectFromServer()));
    close_server_id = tools_menu->insertItem("Close Server...", this, SLOT(closeServer()));

    tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
    tools_menu->setItemEnabled(close_server_id,FALSE);


    help_menu->insertItem(QIconSet(QPixmap(small_icon_xpm)), "About TundraDraw", this, SLOT(about()));

    main_menu->insertItem("&File", file_menu);
    main_menu->insertItem("&Edit", edit_menu);
    main_menu->insertItem("&View", view_menu);
    main_menu->insertItem("&Tools", tools_menu);
    main_menu->insertItem("&Help", help_menu);

    splash->message("Creating controls...", AlignBottom);

    palette_bar = new QToolBar(this);
    palette_bar->setLabel("Colors");
    picker_bar = new QToolBar(this);
    picker_bar->setLabel("Palette");

    charpreview_bar = new QToolBar(this);
    charpreview_bar->setLabel("Character Preview");

    setDockEnabled(charpreview_bar,Right,true);
    setDockEnabled(charpreview_bar,Left,true);
    setDockEnabled(charpreview_bar,Bottom,true);
    setDockEnabled(charpreview_bar,TornOff,true);

    setDockEnabled(palette_bar,Right,true);
    setDockEnabled(palette_bar,Left,true);
    setDockEnabled(palette_bar,Bottom,true);
    setDockEnabled(palette_bar,TornOff,true);

    setDockEnabled(picker_bar, Right, true);
    setDockEnabled(picker_bar, Left, true);
    setDockEnabled(picker_bar, Bottom, true);
    setDockEnabled(picker_bar, TornOff, true);

    TundraPalette* tundra_palette = new TundraPalette(model, picker_bar);
    tundra_palette->show();

    TundraCharPreview* tundra_charpreview = new TundraCharPreview(model, charpreview_bar);
    tundra_charpreview->show();

    QWidget* color_boxes = new QWidget(palette_bar);

    color_boxes->setFixedSize(75,75);

    bgcolor_box = new TundraCurrentColorBox(BACKGROUND, model, color_boxes);
    bgcolor_box->setFrameShape(QFrame::Panel);
    bgcolor_box->setFrameShadow(QFrame::Plain);
    bgcolor_box->setBackgroundColor(BLACK);
    bgcolor_box->setGeometry(30, 30, 30, 30);

    fgcolor_box = new TundraCurrentColorBox(FOREGROUND, model, color_boxes);
    fgcolor_box->setFrameShape(QFrame::Panel);
    fgcolor_box->setFrameShadow(QFrame::Plain);
    fgcolor_box->setBackgroundColor(WHITE);
    fgcolor_box->setGeometry(15, 15, 30, 30);

    canvas_group = new TundraCanvasGroup(model, command_processor, 640, 400, this);
    canvas_group->setFrameShape(QFrame::WinPanel);
    canvas_group->setFrameShadow(QFrame::Sunken);
    canvas_group->setMinimumSize(660,400);
    canvas_group->setFixedWidth(660);

    setCentralWidget(canvas_group);

    chat_bar = new QToolBar(this);
    chat_window = new TundraChatWindow(Qt::Vertical, chat_bar);
    chat_bar->resize(500,140);
    chat_bar->setResizeEnabled(true);
    chat_bar->setStretchableWidget(chat_window);
    chat_bar->setLabel("Chat");


    setDockEnabled(chat_bar,Right,false);
    setDockEnabled(chat_bar,Left,false);
    setDockEnabled(chat_bar,Bottom,true);
    setDockEnabled(chat_bar,TornOff,true);

    moveDockWindow(chat_bar, DockBottom);

    chat_bar->hide();

    user_bar = new QToolBar(this);
    user_bar->hide();
    user_list = new TundraUserList(user_bar);
    user_bar->resize(100,140);
    user_bar->setResizeEnabled(true);
    user_bar->setStretchableWidget(user_list);
    user_bar->setLabel("Users");

    setDockEnabled(user_bar,Right,true);
    setDockEnabled(user_bar,Left,true);
    setDockEnabled(user_bar,Bottom,true);
    setDockEnabled(user_bar,TornOff,true);

    moveDockWindow(user_bar, DockBottom);

    user_bar->hide();

    status_bar = new QStatusBar(this);

    status_bar->message("Ready");

    splash->message("Creating preview window...", AlignBottom);

    preview = new TundraPreview(model, command_processor, this);

    QAccel* a = new QAccel(this);
    a->connectItem(a->insertItem(Key_X+ALT), this, SLOT(swapFgBg()));

    TundraCanvas* canvas = canvas_group->getCanvas();

    clip_board = QApplication::clipboard();

    QObject::connect(canvas, SIGNAL(copying()), this, SLOT(copyingOn()));
    QObject::connect(canvas, SIGNAL(moving()), this, SLOT(movingOn()));
    QObject::connect(canvas, SIGNAL(copyingDone()), this, SLOT(copyingOff()));
    QObject::connect(canvas, SIGNAL(movingDone()), this, SLOT(movingOff()));
    QObject::connect(canvas, SIGNAL(fillRequested()), this, SLOT(showFillMenu()));
    QObject::connect(model, SIGNAL(fgColorChanged()), this, SLOT(fgColorChanged()));
    QObject::connect(model, SIGNAL(bgColorChanged()), this, SLOT(bgColorChanged()));
    QObject::connect(model, SIGNAL(positionChanged()), this, SLOT(cursorMoved()));
    QObject::connect(model, SIGNAL(colorModeChanged()), this, SLOT(colorModeChanged()));
    QObject::connect(model, SIGNAL(warnNewDocument()), this, SLOT(newDocumentSignaled()));
    QObject::connect(model, SIGNAL(newNetworkDocument()), this, SLOT(newDocumentFromNetwork()));
    QObject::connect(model, SIGNAL(selectionOn()), this, SLOT(selectionOn()));
    QObject::connect(model, SIGNAL(selectionOff()), this, SLOT(selectionOff()));
    QObject::connect(preview, SIGNAL(previewHidden()), this, SLOT(vgaPreviewHidden()));
    QObject::connect(command_processor, SIGNAL(canUndo()), this, SLOT(canUndo()));
    QObject::connect(command_processor, SIGNAL(cantUndo()), this, SLOT(cantUndo()));
    QObject::connect(command_processor, SIGNAL(canRedo()), this, SLOT(canRedo()));
    QObject::connect(command_processor, SIGNAL(cantRedo()), this, SLOT(cantRedo()));
    QObject::connect(clip_board, SIGNAL(dataChanged()), this, SLOT(clipboardChanged()));

    if(filename.length() > 0) {
        model->newDocument();
        model->openDocument(filename);
        canvas_group->resetCanvas();
        command_processor->reset();
        setCaption("TundraDraw - "+filename);
    }

    QTimer *timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), this, SLOT(hideSplash()) );

    splash->message("Finished!", AlignBottom);
    timer->start( 1000, TRUE ); // 2 seconds single-shot timer
}

void TundraDrawWindow::setCaption(QString e) {
    caption = e;
    QMainWindow::setCaption(e);
}

void TundraDrawWindow::canUndo() {
    if(caption.contains("*",TRUE)==0) {
        orig_caption=caption;
        this->setCaption(caption+" * ");
    }

    if(save_item != -1) {
        file_menu->setItemEnabled(save_item, true);
    }

    edit_menu->setItemEnabled(undo_item_id, TRUE);
}

void TundraDrawWindow::resizeEvent(QResizeEvent* e) {
    QMainWindow::resizeEvent(e);
    canvas_group->forwardResize();
}

void TundraDrawWindow::vgaPreviewHidden() {
    view_menu->setItemChecked(preview_item_id, false);
}

void TundraDrawWindow::swapFgBg() {
    if(model->getColorMode() == COLOR_24BIT || (model->getColorMode() == COLOR_4BIT && model->getCurrentFg() < 8)) {
        QColor fg_color = model->getPalette(model->getCurrentFg());
        QColor bg_color = model->getPalette(model->getCurrentBg());

        int bg_index = model->getCurrentBg();
        int fg_index = model->getCurrentFg();

        model->setCurrentFg(bg_index);
        model->setCurrentBg(fg_index);

        fgcolor_box->setBackgroundColor(bg_color);
        bgcolor_box->setBackgroundColor(fg_color);
    }
}

void TundraDrawWindow::fgColorChanged() {
    fgcolor_box->setBackgroundColor(model->getPalette(model->getCurrentFg()));
}

void TundraDrawWindow::bgColorChanged() {
    bgcolor_box->setBackgroundColor(model->getPalette(model->getCurrentBg()));
}


void TundraDrawWindow::newDocumentSignaled() {
    // New document is coming, prompt for save
    if(file_menu->isItemEnabled(save_item)) {
        QMessageBox mb( "TundraDraw - Save current file?",
                        "A network user has loaded a new document.\nDo you want to save the current file?",
                        QMessageBox::Question,
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No, QMessageBox::NoButton,
                        this);
        switch( mb.exec() ) {
        case QMessageBox::Yes:
            // save and exit
            saveDocument();
            break;
        }
    }
}

void TundraDrawWindow::newDocumentFromNetwork() {
    // New document received from network
    canvas_group->resetCanvas();

    if(save_item != -1) {
        file_menu->setItemEnabled(save_item, false);
    }

    command_processor->reset();

    setCaption("TundraDraw - Untitled Document");
}

void TundraDrawWindow::newDocument() {
    if(file_menu->isItemEnabled(save_item)) {
        QMessageBox mb( "TundraDraw - Save current file?",
                        "Do you want to save the current file?",
                        QMessageBox::Question,
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No,
                        QMessageBox::Cancel | QMessageBox::Escape,
                        this);
        switch( mb.exec() ) {
        case QMessageBox::Yes:
            // save and exit
            saveDocument();
            model->newDocument();
            canvas_group->resetCanvas();
            if(client_socket != NULL)
                client_socket->sendAnsi();
            if(socket_server != NULL)
                socket_server->sendAnsiToAll();
            if(save_item != -1) {
                file_menu->setItemEnabled(save_item, false);
            }
            command_processor->reset();
            setCaption("TundraDraw - Untitled Document");
            break;
        case QMessageBox::No:
            // exit without saving
            model->newDocument();
            canvas_group->resetCanvas();
            if(client_socket != NULL)
                client_socket->sendAnsi();
            if(socket_server != NULL)
                socket_server->sendAnsiToAll();
            if(save_item != -1) {
                file_menu->setItemEnabled(save_item, false);
            }
            command_processor->reset();
            setCaption("TundraDraw - Untitled Document");
            break;
        case QMessageBox::Cancel:
            // don't save and don't exit
            break;
        }
    }
    else {
        model->newDocument();
        canvas_group->resetCanvas();
        if(client_socket != NULL)
            client_socket->sendAnsi();
        if(socket_server != NULL)
            socket_server->sendAnsiToAll();
        if(save_item != -1) {
            file_menu->setItemEnabled(save_item, false);
        }

        command_processor->reset();

        setCaption("TundraDraw - Untitled Document");
    }
}

void TundraDrawWindow::openDocument() {
    QString f;

    QString filter = "All ANSi Files (*.ans *.asc *.tnd);;Any files (*)";

    if(file_menu->isItemEnabled(save_item)) {
        QMessageBox mb( "TundraDraw - Save current file?",
                        "Do you want to save the current file?",
                        QMessageBox::Question,
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No,
                        QMessageBox::Cancel | QMessageBox::Escape,
                        this);
        switch( mb.exec() ) {
        case QMessageBox::Yes:
            // save and exit
            saveDocument();
            f = QFileDialog::getOpenFileName( QString::null, filter, this );
            if ( !f.isEmpty() ) {
                // the user selected a valid existing file
                model->newDocument();
                model->openDocument(f);
                canvas_group->resetCanvas();
                if(client_socket != NULL)
                    client_socket->sendAnsi();
                if(socket_server != NULL)
                    socket_server->sendAnsiToAll();
                command_processor->reset();
                setCaption("TundraDraw - "+f);
            } else {
                // the user cancelled the dialog
            }
            break;
        case QMessageBox::No:
            // exit without saving
            f = QFileDialog::getOpenFileName( QString::null, filter, this );
            if ( !f.isEmpty() ) {
                // the user selected a valid existing file
                model->newDocument();
                model->openDocument(f);
                canvas_group->resetCanvas();
                if(client_socket != NULL)
                    client_socket->sendAnsi();
                if(socket_server != NULL)
                    socket_server->sendAnsiToAll();
                command_processor->reset();
                setCaption("TundraDraw - "+f);
                if(save_item != -1) {
                    file_menu->setItemEnabled(save_item, false);
                }
            } else {
                // the user cancelled the dialog
            }
            break;
        case QMessageBox::Cancel:
            // don't save and don't exit
            break;
        }

    }
    else {
        f = QFileDialog::getOpenFileName( QString::null, filter, this );
        if ( !f.isEmpty() ) {
            // the user selected a valid existing file
            model->newDocument();
            model->openDocument(f);
            canvas_group->resetCanvas();
            if(client_socket != NULL)
                client_socket->sendAnsi();
            if(socket_server != NULL)
                socket_server->sendAnsiToAll();
            if(save_item != -1) {
                file_menu->setItemEnabled(save_item, false);
            }
            command_processor->reset();
            setCaption("TundraDraw - "+f);
        } else {
            // the user cancelled the dialog
        }
    }

    if(model->getColorMode() == COLOR_24BIT) {
        colorMode24Bit();
    }
    else {
        colorMode4Bit();
    }
}

void TundraDrawWindow::saveDocument() {
    if(!model->hasFilename()) {
        // start at the current working directory and with *.ans as filter
        QString filter;

        if(model->getColorMode() == COLOR_24BIT)
            filter = "TundraDraw files (*.tnd)";
        else
            filter = "ANSi Files (*.ans *.asc)";

        QString f = QFileDialog::getSaveFileName( QString::null, filter, this );
        if ( !f.isEmpty() ) {
            TundraSaveStrategy* save_strategy = NULL;

            // the user gave a file name
            if(model->getColorMode() == COLOR_24BIT) {
                save_strategy = new TundraSave24BitStrategy();
            }
            else if(f.endsWith(".asc")) {
                // Save as ASCII
                save_strategy = new TundraSaveAsciiStrategy();
            }
            else {
                // Save as regular ANSi
                save_strategy = new TundraSaveAnsiStrategy();
            }

            if(!f.contains('.')) {
                // Append file extension
                if(model->getColorMode() == COLOR_24BIT) {
                    f += QString(".tnd");
                }
                else if(model->getColorMode() == COLOR_4BIT) {
                    f += QString(".ans");
                }
            }

            if(model->saveDocument(f, save_strategy)) {
                // The save was successful
                orig_caption = "TundraDraw - "+f;
                setCaption(orig_caption);
                if(save_item != -1) {
                    file_menu->setItemEnabled(save_item, false);
                }
            }
            else {
                QMessageBox::warning( this, "Save failed!",
                                      QString("Couldn't save file:"+f+"!"));
            }

        } else {
            // the user cancelled the dialog
        }
    }
    else {
        if(model->saveDocument()) {
            // The save was successful
            if(save_item != -1) {
                file_menu->setItemEnabled(save_item, false);
            }
            this->setCaption(orig_caption);
        }
        else {
            QMessageBox::warning( this, "Save failed!",
                                  QString("Couldn't save file!"));
        }
    }
}

void TundraDrawWindow::saveDocumentAs() {
    QString filter;

    if(model->getColorMode() == COLOR_24BIT)
        filter = "TundraDraw files (*.tnd)";
    else
        filter = "ANSi Files (*.ans *.asc)";

    QString f = QFileDialog::getSaveFileName( QString::null, filter, this );
    if ( !f.isEmpty() ) {
        TundraSaveStrategy* save_strategy = NULL;

        if(model->getColorMode() == COLOR_24BIT) {
            save_strategy = new TundraSave24BitStrategy();
        }
        // the user gave a file name
        else if(f.endsWith(".asc")) {
            // Save as ASCII
            save_strategy = new TundraSaveAsciiStrategy();
        }
        else {
            // Save as regular ANSi
            save_strategy = new TundraSaveAnsiStrategy();
        }

        if(!f.contains('.')) {
            // Append file extension
            if(model->getColorMode() == COLOR_24BIT) {
                f += QString(".tnd");
            }
            else if(model->getColorMode() == COLOR_4BIT) {
                f += QString(".ans");
            }
        }

        // the user gave a file name
        if(model->saveDocument(f, save_strategy)) {
            // The save was successful
            orig_caption = "TundraDraw - "+f;
            setCaption(orig_caption);
            if(save_item != -1) {
                file_menu->setItemEnabled(save_item, false);
            }
        }
        else {
            QMessageBox::warning( this, "Save failed!",
                                  QString("Couldn't save file:"+f+"!"));
        }

    } else {
        // the user cancelled the dialog
    }
}

void TundraDrawWindow::cursorMoved() {
    QPoint position = model->getCurrentPosition();
    status_bar->message("Ready - "+QString("%1").arg(position.x())+", "+QString("%1").arg(position.y()));
}

void TundraDrawWindow::about() {
    TundraAboutDialog dlg(this);
    dlg.exec();
}

void TundraDrawWindow::fillAttribute() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraFillAttributeStrategy* strategy = new TundraFillAttributeStrategy(model->getCurrentFg(), model->getCurrentBg());
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::fillCharacter() {

    TundraFillCharDlg* dlg = new TundraFillCharDlg(model, this);
    if(dlg->exec() == QDialog::Accepted) {
        unsigned char character = 0;
        int fg_color;

        if(model->getColorMode() == COLOR_4BIT)
            fg_color=7;
        else
            fg_color = packColor(LT_WHITE.red(), LT_WHITE.green(), LT_WHITE.blue());

        int bg_color=0;
        QPoint top_left;
        QPoint bottom_right;
        model->getSelection(top_left, bottom_right);
        if(dlg->getKey() < 128 && dlg->getKey() > 31) {
            character = dlg->getKey();
        }
        else if(dlg->getKey() >= 0x1030 && dlg->getKey() < 0x103a) {
            character = char_set[model->getCharSet()][(dlg->getKey() - 0x1030)];
        }
        // Here we want to fill the current selection with the requested character
        TundraFillCharStrategy* strategy = new TundraFillCharStrategy(character, fg_color, bg_color);
        TundraBlockCommand* block_command = command_processor->newBlockCommand(top_left, bottom_right, strategy);
        //command = block_command;
        command_processor->process(block_command);
        model->cancelSelection();
    }
}

void TundraDrawWindow::fillCharacterAttribute() {

    TundraFillCharDlg* dlg = new TundraFillCharDlg(model, this);
    if(dlg->exec() == QDialog::Accepted) {
        unsigned char character = 0;
        int fg_color=model->getCurrentFg();
        int bg_color=model->getCurrentBg();
        QPoint top_left;
        QPoint bottom_right;
        model->getSelection(top_left, bottom_right);
        if(dlg->getKey() < 128 && dlg->getKey() > 31) {
            character = dlg->getKey();
        }
        else if(dlg->getKey() >= 0x1030 && dlg->getKey() < 0x103a) {
            character = char_set[model->getCharSet()][dlg->getKey() - 0x1030];
        }
        // Here we want to fill the current selection with the requested character
        TundraFillBothStrategy* strategy = new TundraFillBothStrategy(character, fg_color, bg_color);
        TundraBlockCommand* block_command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

        command_processor->process(block_command);
        model->cancelSelection();
    }
}

void TundraDrawWindow::fillFgColor() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraFillFgStrategy* strategy = new TundraFillFgStrategy(model->getCurrentFg());
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::fillBgColor() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraFillBgStrategy* strategy = new TundraFillBgStrategy(model->getCurrentBg());
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::erase() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraEraseStrategy* strategy = new TundraEraseStrategy();
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::copy() {
    canvas_group->getCanvas()->setCopying();
}

void TundraDrawWindow::move() {
    canvas_group->getCanvas()->setMoving();
}

void TundraDrawWindow::viewPreviewWindow() {
    if(preview->isVisible()) {
        preview->hide();
        view_menu->setItemChecked(preview_item_id, false);
    }
    else {
        preview->show();
        view_menu->setItemChecked(preview_item_id, true);
    }
}

void TundraDrawWindow::startServer() {
    TundraStartServerDlg dlg(this);
    if(dlg.exec() == QDialog::Accepted) {
        // Start the server
        socket_server = new TundraServerSocket(chat_window, model, dlg.getNick(), dlg.getPort());
        net_command_visitor->setServer(socket_server);
        chat_bar->show();
        user_bar->show();
        tools_menu->setItemEnabled(start_server_id,FALSE);
        tools_menu->setItemEnabled(close_server_id,TRUE);
        tools_menu->setItemEnabled(connect_to_server_id,FALSE);
        tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
        user_list->addUser(dlg.getNick());
        chat_window->showUserLogin(dlg.getNick());
        chat_window->setNick(dlg.getNick());
        QObject::connect(socket_server, SIGNAL(userLoggedIn(QString)), user_list, SLOT(addUser(QString)));
        QObject::connect(socket_server, SIGNAL(userLoggedIn(QString)), chat_window,
                         SLOT(showUserLogin(QString)));
        QObject::connect(socket_server, SIGNAL(userLoggedOut(QString)), user_list, SLOT(removeUser(QString)));
        QObject::connect(socket_server, SIGNAL(userLoggedOut(QString)), chat_window,
                         SLOT(showUserPart(QString)));
        QObject::connect(chat_window, SIGNAL(messageWritten(QString)), socket_server, SLOT(sendTextMessage(QString)));
    }
}

void TundraDrawWindow::connectToServer() {
    TundraConnectDlg dlg(this);
    if(dlg.exec() == QDialog::Accepted) {
        // Connect to the server
        client_socket = new TundraClientSocket(chat_window, user_list, model, dlg.getNick());
        client_socket->connectToServer(dlg.getIp(), dlg.getPort());
        net_command_visitor->setClient(client_socket);
        chat_window->setNick(dlg.getNick());
        QObject::connect(chat_window, SIGNAL(messageWritten(QString)), client_socket, SLOT(sendTextMessage(QString)));
        QObject::connect(client_socket, SIGNAL(connectionClosed()), chat_window, SLOT(showDisconnected()));
        QObject::connect(client_socket, SIGNAL(connectionClosed()), user_list, SLOT(clear()));
        QObject::connect(client_socket, SIGNAL(connectionClosed()), net_command_visitor, SLOT(setDisconnected()));
        QObject::connect(client_socket, SIGNAL(error(int)), this, SLOT(clientSocketError(int) ) );
        chat_bar->show();
        user_bar->show();
        tools_menu->setItemEnabled(start_server_id,FALSE);
        tools_menu->setItemEnabled(close_server_id,FALSE);
        tools_menu->setItemEnabled(connect_to_server_id,FALSE);
        tools_menu->setItemEnabled(disconnect_from_server_id,TRUE);
    }
}

void TundraDrawWindow::clientSocketError(int e) {
    QString error_message;

    if(e == QSocket::ErrConnectionRefused)
        error_message = "The connection was refused";
    else if(e == QSocket::ErrHostNotFound)
        error_message = "The host could not be found";
    else if(e == QSocket::ErrSocketRead)
        error_message = "Problem reading from socket";
    else
        error_message = "Unknown network error";

    QMessageBox::warning(this, "Connection Problem", error_message);
    chat_window->showDisconnected();
    user_list->clear();
    net_command_visitor->setDisconnected();

    delete client_socket;
    client_socket = NULL;

    chat_bar->hide();
    user_bar->hide();

    tools_menu->setItemEnabled(start_server_id,TRUE);
    tools_menu->setItemEnabled(close_server_id,FALSE);
    tools_menu->setItemEnabled(connect_to_server_id,TRUE);
    tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
}

void TundraDrawWindow::disconnectFromServer() {
    /* Disconnect from server */
    QMessageBox mb( "TundraDraw - Confirm Disconnect",
                    "Are you sure you would like to disconnect from the server?",
                    QMessageBox::Question,QMessageBox::Yes,QMessageBox::No |
                    QMessageBox::Default, 0, this);
    switch( mb.exec() ) {
    case QMessageBox::Yes:
        if(client_socket != NULL) {
            client_socket->disconnectFromServer();
            delete client_socket;
            client_socket = NULL;

            net_command_visitor->setDisconnected();

            chat_window->showDisconnected();
            user_list->clear();
            chat_bar->hide();
            user_bar->hide();
            tools_menu->setItemEnabled(start_server_id,TRUE);
            tools_menu->setItemEnabled(close_server_id,FALSE);
            tools_menu->setItemEnabled(connect_to_server_id,TRUE);
            tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
        }
        tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
        tools_menu->setItemEnabled(close_server_id,FALSE);
        break;
    case QMessageBox::No:
        // don't disconnect
        break;
    case QMessageBox::Cancel:
        // don't disconnect
        break;
    }
}

void TundraDrawWindow::closeServer() {
    QMessageBox mb( "TundraDraw - Confirm Disconnect",
                    "Are you sure you would like to close the server?\nThis will disconnect ALL clients!",
                    QMessageBox::Warning,QMessageBox::Yes,QMessageBox::No |
                    QMessageBox::Default, 0, this);
    switch( mb.exec() ) {
    case QMessageBox::Yes:
        if(socket_server != NULL) {
            socket_server->closeServer();
            delete socket_server;
            socket_server = NULL;

            net_command_visitor->setDisconnected();

            chat_window->showDisconnected();
            user_list->clear();
            chat_bar->hide();
            user_bar->hide();
            tools_menu->setItemEnabled(start_server_id,TRUE);
            tools_menu->setItemEnabled(close_server_id,FALSE);
            tools_menu->setItemEnabled(connect_to_server_id,TRUE);
            tools_menu->setItemEnabled(disconnect_from_server_id,FALSE);
        }
        break;
    case QMessageBox::No:
        // don't disconnect
        break;
    case QMessageBox::Cancel:
        // don't disconnect
        break;
    }
}

void TundraDrawWindow::undo() {
    command_processor->undo();
}

void TundraDrawWindow::redo() {
    command_processor->redo();
}

void TundraDrawWindow::colorMode4Bit() {
    bool do_change = true;

    if(model->getColorMode() != COLOR_4BIT) {
        int result = QMessageBox::warning(this, "Warning!", "Converting back to 4-bit ANSi will cause a loss of color data.\nAre you sure you want to continue?", QMessageBox::Yes, QMessageBox::No);
        if(result != QMessageBox::Yes)
            do_change = false;

    }

    if(do_change) {
        mode_menu->setItemChecked(mode_4bit_id, true);
        mode_menu->setItemChecked(mode_24bit_id, false);
        if(model->getColorMode() != COLOR_4BIT) {
            model->setColorMode(COLOR_4BIT);
            if(client_socket != NULL)
                client_socket->sendColorMode();
            if(socket_server != NULL)
                socket_server->sendColorModeToAll();
        }
    }
    // Check if we have anything we can paste in this new mode
    clipboardChanged();
}

void TundraDrawWindow::colorMode24Bit() {
    mode_menu->setItemChecked(mode_4bit_id, false);
    mode_menu->setItemChecked(mode_24bit_id, true);
    if(model->getColorMode() != COLOR_24BIT) {
        model->setColorMode(COLOR_24BIT);
        if(client_socket != NULL)
            client_socket->sendColorMode();
        if(socket_server != NULL)
            socket_server->sendColorModeToAll();
    }
    // Check if we have anything we can paste in this new mode
    clipboardChanged();
}

void TundraDrawWindow::cantUndo() {
    edit_menu->setItemEnabled(undo_item_id, FALSE);
    this->setCaption(orig_caption);

    if(save_item != -1) {
        file_menu->setItemEnabled(save_item, false);
    }

}

void TundraDrawWindow::hideSplash() {
    splash->finish(this);
    delete splash;
}

void TundraDrawWindow::closeEvent(QCloseEvent* e) {
    if(file_menu->isItemEnabled(save_item)) {
        QMessageBox mb( "TundraDraw - Save current file?",
                        "Do you want to save the current file before exiting?",
                        QMessageBox::Question,
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No,
                        QMessageBox::Cancel | QMessageBox::Escape,
                        this);
        switch( mb.exec() ) {
        case QMessageBox::Yes:
            // save and exit
            saveDocument();
            if(!file_menu->isItemEnabled(save_item))
                e->accept();
            break;
        case QMessageBox::No:
            // exit without saving
            e->accept();
            break;
        case QMessageBox::Cancel:
            // don't save and don't exit
            break;
        }
    }
    else {
        e->accept();
    }
}

void TundraDrawWindow::selectionOn() {
    edit_menu->setItemEnabled(block_id, TRUE);
    edit_menu->setItemEnabled(copy_id, TRUE);
    edit_menu->setItemEnabled(move_id, TRUE);
    edit_menu->setItemEnabled(copy_clipboard_id, TRUE);
    edit_menu->setItemEnabled(cut_clipboard_id, TRUE);
    status_bar->message("Block Mode - (M)ove / (C)opy / (E)rase / (F)ill / Flip (X) / Flip (Y)");
}

void TundraDrawWindow::selectionOff() {
    edit_menu->setItemEnabled(block_id, FALSE);
    edit_menu->setItemEnabled(copy_id, FALSE);
    edit_menu->setItemEnabled(move_id, FALSE);
    edit_menu->setItemEnabled(copy_clipboard_id, FALSE);
    edit_menu->setItemEnabled(cut_clipboard_id, FALSE);
    status_bar->message("Ready");
}

void TundraDrawWindow::flipX() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraFlipXStrategy* strategy = new TundraFlipXStrategy();
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::flipY() {
    QPoint top_left, bottom_right;
    model->getSelection(top_left, bottom_right);

    TundraFlipYStrategy* strategy = new TundraFlipYStrategy();
    TundraBlockCommand* command = command_processor->newBlockCommand(top_left, bottom_right, strategy);

    model->cancelSelection();
    command_processor->process(command);
}

void TundraDrawWindow::copyingOn() {
    status_bar->message("Copying - Arrow keys to move / (S)tamp / P or Spacebar to Paste");
}

void TundraDrawWindow::movingOn() {
    status_bar->message("Moving - Arrow keys to move / Spacebar to place ");
}

void TundraDrawWindow::copyingOff() {
    status_bar->message("Ready");
}

void TundraDrawWindow::movingOff() {
    status_bar->message("Ready");
}

void TundraDrawWindow::pasteFromClipboard() {
    QMimeSource* data = clip_board->data();
    if((data->provides("image/tundradraw") && model->getColorMode() == COLOR_24BIT) || data->provides("PABLODRAW_BUFFER")) {
        int width;
        int height;

        QByteArray encoded_data;

        if((data->provides("image/tundradraw") && model->getColorMode() == COLOR_24BIT))
            encoded_data = data->encodedData("image/tundradraw");
        else
            encoded_data = data->encodedData("PABLODRAW_BUFFER");

        QDataStream is(encoded_data, IO_ReadOnly);
        char* width_array = (char*)&width;
        char* height_array = (char*)&height;

        is.readRawBytes(width_array, 4);
        is.readRawBytes(height_array, 4);

        char* array_data = encoded_data.data();
        unsigned int size = encoded_data.size();

        QByteArray new_data;
        new_data.duplicate(array_data+8, size-8);

        TundraDoc** doc = new TundraDoc*[height];
        doc[0] = new TundraDoc[width*height];
        for(int i = 1; i < height; i++) {
            doc[i] = doc[0]+(i*width);
        }

        for(int i = 0; i < height; i++) {
            for(int j = 0; j < width; j++) {
                doc[i][j].character = 0;
                doc[i][j].fg_color = 0;
                doc[i][j].bg_color = 0;
            }
        }

        QFile null_file;
        if((data->provides("image/tundradraw") && model->getColorMode() == COLOR_24BIT))
            model->open24BitANSi(null_file, &new_data, doc, width, height);
        else
            model->openANSi(null_file, &new_data, doc, width, height);

        QPoint start_position = model->getCurrentPosition();
        QPoint end_position(start_position.x()+width, start_position.y()+height);

        TundraPasteStrategy* strategy = new TundraPasteStrategy(doc, width, height);
        TundraBlockCommand* block_command = command_processor->newBlockCommand(start_position, end_position, strategy);

        command_processor->process(block_command);
    }
    else {
        QMessageBox::information(this, "Paste", "FALSE!");
    }
}

void TundraDrawWindow::copyToClipboard() {
    TundraMimeSource* mime_source = new TundraMimeSource(model);
    clip_board->setData(mime_source);
    model->cancelSelection();
}

void TundraDrawWindow::cutToClipboard() {
    TundraMimeSource* mime_source = new TundraMimeSource(model);
    clip_board->setData(mime_source);

    erase();
}

void TundraDrawWindow::clipboardChanged() {
    QMimeSource* data = clip_board->data();
    if(data->provides("PABLODRAW_BUFFER") || (model->getColorMode() == COLOR_24BIT && data->provides("image/tundradraw"))) {
        edit_menu->setItemEnabled(paste_clipboard_id, TRUE);
    }
    else {
        edit_menu->setItemEnabled(paste_clipboard_id, FALSE);
    }
}

void TundraDrawWindow::showFillMenu() {
    TundraCanvas* canvas = canvas_group->getCanvas();
    QPoint pos = model->getCurrentPosition();
    int width = canvas->width();
    width /= 80;
    int height = width * 2;

    key_fill_menu->popup(canvas->mapToGlobal(QPoint(pos.x() * width, pos.y() * height)));
}

void TundraDrawWindow::exportUnitTestData() {
    QString f = QFileDialog::getSaveFileName(QString::null, "*.dat", this);
    if(!f.isNull()) {
        if(model->exportUnitTestData(f)) {
            QMessageBox::information(this, "Success!", "Unit test data successfully saved");
        } else {
            QMessageBox::critical(this, "Error!", "Could not save unit test data!");
        }
    }
}

void TundraDrawWindow::importUnitTestData24() {
    QString f = QFileDialog::getOpenFileName(QString::null, "*.dat", this);
    if(!f.isNull()) {
        if(model->importUnitTestData(f, COLOR_24BIT)) {
            QMessageBox::information(this, "Success!", "Unit test data successfully imported");
        } else {
            QMessageBox::critical(this, "Error!", "Could not save unit test data!");
        }
    }
}

void TundraDrawWindow::importUnitTestData4() {
    QString f = QFileDialog::getOpenFileName(QString::null, "*.dat", this);
    if(!f.isNull()) {
        if(model->importUnitTestData(f, COLOR_4BIT)) {
            QMessageBox::information(this, "Success!", "Unit test data successfully imported");
        } else {
            QMessageBox::critical(this, "Error!", "Could not save unit test data!");
        }
    }
}
