/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrafillchardlg.h"

#include <qvariant.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/*
 *  Constructs a TundraServerDlg as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
TundraFillCharDlg::TundraFillCharDlg( TundraModel* model, QWidget* parent, const char* name, bool modal, WFlags fl )
        : QDialog( parent, name, modal, fl )
{
    if ( !name )
        setName( "TundraFillCharDlg" );

    fill_label = new QLabel( this, "fill_label" );
    fill_label->setGeometry( QRect( 15, 20, 111, 16 ) );
    char_preview = new TundraCharPreviewWidgetFrame(model,32, false, this);
    char_preview->setGeometry( QRect(140,19,13,20) );
    ok_button = new QPushButton( this, "ok_button" );
    ok_button->setGeometry( QRect( 165, 14, 51, 31 ) );
    ok_button->setOn( FALSE );
    ok_button->setDefault( TRUE );
    resize( QSize(234, 65).expandedTo(minimumSizeHint()) );
    languageChange();
    clearWState( WState_Polished );
    QObject::connect(this, SIGNAL(keyUpdated(int)), char_preview, SLOT(updateKey(int)));
    QObject::connect(ok_button, SIGNAL(released()), this, SLOT(checkFields()));
}

/*
 *  Destroys the object and frees any allocated resources
 */
TundraFillCharDlg::~TundraFillCharDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void TundraFillCharDlg::languageChange()
{
    setCaption( tr( "Fill With Character" ) );
    fill_label->setText( tr( "Character to use:" ) );
    ok_button->setText( tr( "Ok" ) );
}

void TundraFillCharDlg::checkFields()
{
    accept();
}
void TundraFillCharDlg::keyPressEvent(QKeyEvent* e)
{

    if(e->ascii() < 128 && e->ascii() > 31) {
        this_key = e->text().at(0).unicode();
        emit keyUpdated(e->text().at(0).unicode());
    }
    else if(e->key() >= 0x1030 && e->key() < 0x103a) { // Function Keys F1-F10
        this_key = e->key();
        emit keyUpdated(e->key());
    } else if(e->key() == 0x1000) // ESC Key
        reject();
    else if(e->key() == Key_Return) {
        accept();
    }
	else if(e->key() != 0x1020) { // SHIFT KEY we do not want to report a debug message here 
        // Unhandled key
	}

}


