/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAFILLCHARDLG_H
#define TUNDRAFILLCHARDLG_H

#include <qvariant.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qpushbutton.h>

#include "tundracharpreviewwidgetframe.h"
#include "tundramodel.h"

//! The fill character dialog
/*!
	The fill character dialog appears when the user chooses to 'Fill Character' or 'Fill Both'.
	It allows the user to press a key on the keyboard to pick the character with which to fill
	and renders that key in a small preview.
*/
class TundraFillCharDlg : public QDialog
{
    Q_OBJECT

public:
	//! Constructor
	/*! 
		\param model A pointer to the model
	*/
    TundraFillCharDlg( TundraModel* model, QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );

	//! Destructor
    ~TundraFillCharDlg();

	//! The label 
    QLabel* fill_label;

	//! The OK button
    QPushButton* ok_button;

	//! Standard Qt keyPressEvent override
	/*!
		We override keyPressEvent so we can get the character with which to fill
	*/
    void keyPressEvent(QKeyEvent* e);

	//! The preview widget
    TundraCharPreviewWidgetFrame* char_preview;

	//! Accessor for the key the user pressed
    int getKey() { return this_key; }

signals:
	//! Signal a new keypress
    void keyUpdated(int new_key);

protected:

protected slots:
	//! Close the dialog. (No validation is currently done)
    void checkFields();

private:
    void languageChange();
    TundraModel* model;
    int this_key;

};

#endif // TUNDRASTARTSERVERDLG_H
