/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrainsertcommand.h"

TundraInsertCommand::TundraInsertCommand(TundraModel* m, TundraDoc& doc, int x, int y) : is_remote(false)
{
    my_x = x;
    my_y = y;

    model = m;

    new_doc.bg_color = doc.bg_color;
    new_doc.fg_color = doc.fg_color;
    new_doc.character = doc.character;
}

TundraInsertCommand::~TundraInsertCommand(void)
{
}

void TundraInsertCommand::execute() {

    TundraDoc* last_loc = model->getItem(model->getMaxX(), my_y);
    last_char_doc.character = last_loc->character;
    last_char_doc.fg_color = last_loc->fg_color;
    last_char_doc.bg_color = last_loc->bg_color;

    TundraDoc* current;
    TundraDoc* last;

    // Move everything over one column
    for(int i = model->getMaxX(); i > my_x; --i) {
        current = model->getItem(i, my_y);
        last = model->getItem(i-1, my_y);

        current->bg_color = last->bg_color;
        current->fg_color = last->fg_color;
        current->character = last->character;
    }

    TundraDoc* loc = model->getItem(my_x, my_y);

    loc->fg_color = new_doc.fg_color;
    loc->bg_color = new_doc.bg_color;
    loc->character = new_doc.character;

    if(!is_remote)
        model->setCurrentPosition(my_x+1, my_y);
    model->setModified();
}

void TundraInsertCommand::unexecute() {

    TundraDoc* current;
    TundraDoc* next;

    for(int i = my_x; i < model->getMaxX()-1; i++) {
        current = model->getItem(i, my_y);
        next = model->getItem(i+1, my_y);

        current->bg_color = next->bg_color;
        current->fg_color = next->fg_color;
        current->character = next->character;
    }

    current = model->getItem(model->getMaxX(), my_y);

    current->bg_color = last_char_doc.bg_color;
    current->fg_color = last_char_doc.fg_color;
    current->character = last_char_doc.character;

    if(!is_remote)
        model->setCurrentPosition(my_x, my_y);
    model->setModified();
}

