/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAINSERTCOMMAND_H
#define TUNDRAINSERTCOMMAND_H

#include "tundracommand.h"
#include "tundramodel.h"
#include "tundrainsertcommand.h"
#include "tundracommandvisitor.h"

//! Place a single block on the canvas, shifting blocks to the right over one column.
/*!
	The insert command is used to place a single block on the canvas when insert mode is on. 
	It directly alters the model.

	\sa TundraCharacterCommand
*/
class TundraInsertCommand :
            public TundraCommand
{
public:
	//! Constructor
	/*!
		\param m A pointer to the model
		\param doc The new block to use
		\param x The x location on which to place the block
		\param y The y location on which to place the block
	*/
    TundraInsertCommand(TundraModel* m, TundraDoc& doc, int x, int y);

	//! Destructor
    ~TundraInsertCommand(void);

	//! Place the new block on the canvas (in the model)
    void execute();

	//! Replace the new block with the original block, and shift the contents to the right back one column
    void unexecute();

	//! Accept a visitor
	/*!
		\param v A pointer to a command visitor
	*/
    void accept(TundraCommandVisitor* v) { v->visitInsertCommand(this); }

	//! Set this for commands coming in over the network
	/*!
		We set this for network commands so we do not move the cursor
	*/
    void setRemote() { is_remote = true; }

	//! The x location on which to place the block
    int my_x;

	//! The y location on which to place the block
    int my_y;

	//! The new block to use
    TundraDoc new_doc;
private:
    TundraDoc last_char_doc;
    TundraModel* model;
    bool is_backspace;
    bool is_remote;
};

#endif
