/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundramimesource.h"

TundraMimeSource::TundraMimeSource(TundraModel* m) : model(m)
{
    model->getSelection(top_left, bottom_right);

    int width = bottom_right.x() - top_left.x();
    int height = bottom_right.y() - top_left.y();

    TundraDoc** doc = new TundraDoc*[height];
    doc[0] = new TundraDoc[width*height];

    TundraSaveStrategy* strategy = NULL;

    my_mode = model->getColorMode();

    if(my_mode == COLOR_4BIT)
        strategy = new TundraSaveAnsiStrategy();
    else if(my_mode == COLOR_24BIT)
        strategy = new TundraSave24BitStrategy();

    strategy->saveFile(QString(), model->getDocument(), model->getMaxY(), &my_data, (QPoint*)&top_left, (QPoint*)&bottom_right);
}

TundraMimeSource::~TundraMimeSource(void)
{
}

const char* TundraMimeSource::format(int i) const {
    if(my_mode == COLOR_4BIT) {
        switch(i) {
        case 0:
            return "PABLODRAW_BUFFER";
            break;
        default:
            return NULL;
            break;
        }
    }
    else if(my_mode == COLOR_24BIT) {
        switch(i) {
        case 0:
            return "image/tundradraw";
            break;
        default:
            return NULL;
            break;
        }
    }
    else
        return NULL;
}

bool TundraMimeSource::provides(const char* mimeType) const {
    QString mime_type = QString(mimeType);
    if(my_mode == COLOR_4BIT) {
        if(mime_type == QString("PABLODRAW_BUFFER")) {
            return TRUE;
        }
    }
    else if(my_mode == COLOR_24BIT) {
        if(mime_type == QString("image/tundradraw")) {
            return TRUE;
        }
    }

    return false;
}

QByteArray TundraMimeSource::encodedData(const char* mimeType) const {
    QString mime_type = QString(mimeType);
    if((my_mode == COLOR_4BIT && mime_type == QString("PABLODRAW_BUFFER")) ||
            (my_mode == COLOR_24BIT && mime_type == QString("image/tundradraw")))
        return my_data;
    else
        return QByteArray();
}
