/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAMIMESOURCE_H
#define TUNDRAMIMESOURCE_H

#include <qmime.h>
#include <qpoint.h>

#include "tundramodel.h"
#include "tundrasaveansistrategy.h"

//! Encode a block for use with clipboard
/*!
    We subclass TundraMimeSource so that we can provide mime encoding data
    for use with the clipboard copy/paste. Note that we use the PABLODRAW_BUFFER
    signature as a mime type for 4-bit ANSI, so that we can retain compatibility
    with PabloDraw.
*/
class TundraMimeSource :
            public QMimeSource
{
public:
    //! Constructor
    /*! 
        \param A pointer to the model
    */
    TundraMimeSource(TundraModel* m);

    //! Destructor
    virtual ~TundraMimeSource(void);

    //! List the formats this mime source supplies
    /*!
        \param i The index of the mime format
        
        \return The mime type as a string, or NULL if the specified index does not exist
    */
    const char* format(int i = 0) const;

    //! Return if this mime source provides the specified mime type
    /*!
        \param mimeType The mime type to check for

        \return TRUE if this class provides the mime type, FALSE otherwise
    */
    bool provides(const char* mimeType) const;

    //! Provided the encoded data in the specified format
    /*!
        \param mimeType The mime type to provide

        \return A QByteArray providing the encoded bytes for the specified format
    */
    QByteArray encodedData(const char* mimeType) const;
private:
    TundraModel* model;
    QPoint top_left;
    QPoint bottom_right;
    QByteArray my_data;
    unsigned char my_mode;
};

#endif
