/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundramodel.h"
#include "graphics.h"
#include <vector>
#include <qbitmap.h>
#include <qvaluevector.h>

using namespace std;

static const char header_24bit[] = "TUNDRA24";

#define COLOR_RED 0
#define COLOR_GREEN 1
#define COLOR_BLUE 2

TundraModel::TundraModel() : has_filename(false), current_position(QPoint(0,0)), last_position(QPoint(0,0)), current_fg(7), current_bg(0),  max_y(0), aspect(40.0), current_save_strategy(NULL)  {
    color_mode = COLOR_4BIT;

    palette[0] = BLACK;
    palette[1] = RED;
    palette[2] = GREEN;
    palette[3] = YELLOW;
    palette[4] = BLUE;
    palette[5] = MAGENTA;
    palette[6] = CYAN;
    palette[7] = WHITE;
    palette[8] = LT_BLACK;
    palette[9] = LT_RED;
    palette[10] = LT_GREEN;
    palette[11] = LT_YELLOW;
    palette[12] = LT_BLUE;
    palette[13] = LT_MAGENTA;
    palette[14] = LT_CYAN;
    palette[15] = LT_WHITE;

    // Block to switch on Flip X
    flip_x_map.insert(221, 222);
    flip_x_map.insert(222, 221);
    flip_x_map.insert(180, 195);
    flip_x_map.insert(195, 180);
    flip_x_map.insert(181, 198);
    flip_x_map.insert(198, 181);
    flip_x_map.insert(182, 199);
    flip_x_map.insert(199, 182);
    flip_x_map.insert(183, 214);
    flip_x_map.insert(214, 183);
    flip_x_map.insert(184, 213);
    flip_x_map.insert(213, 184);
    flip_x_map.insert(185, 204);
    flip_x_map.insert(204, 185);
    flip_x_map.insert(187, 201);
    flip_x_map.insert(201, 187);
    flip_x_map.insert(188, 200);
    flip_x_map.insert(200, 188);
    flip_x_map.insert(189, 211);
    flip_x_map.insert(211, 189);
    flip_x_map.insert(190, 212);
    flip_x_map.insert(212, 190);
    flip_x_map.insert(191, 218);
    flip_x_map.insert(218, 191);
    flip_x_map.insert(192, 217);
    flip_x_map.insert(217, 192);

    // Blocks to switch on Flip Y
    flip_y_map.insert(220, 223);
    flip_y_map.insert(223, 220);
    flip_y_map.insert(183, 189);
    flip_y_map.insert(189, 183);
    flip_y_map.insert(184, 190);
    flip_y_map.insert(190, 184);
    flip_y_map.insert(187, 188);
    flip_y_map.insert(188, 187);
    flip_y_map.insert(191, 217);
    flip_y_map.insert(217, 191);
    flip_y_map.insert(192, 218);
    flip_y_map.insert(218, 192);
    flip_y_map.insert(193, 194);
    flip_y_map.insert(194, 193);
    flip_y_map.insert(200, 201);
    flip_y_map.insert(201, 200);
    flip_y_map.insert(202, 203);
    flip_y_map.insert(203, 202);
    flip_y_map.insert(207, 209);
    flip_y_map.insert(209, 207);
    flip_y_map.insert(208, 210);
    flip_y_map.insert(210, 208);
    flip_y_map.insert(211, 214);
    flip_y_map.insert(214, 211);
    flip_y_map.insert(212, 213);
    flip_y_map.insert(213, 212);

    for(int i = 0; i < 256; i++) {
        characters[i] = QBitmap(9, 16, &graphics9[i*32]);
    }

    document = new TundraDoc*[NUM_ROWS];
    document[0] = new TundraDoc[NUM_ROWS*NUM_COLS];

    for(int r = 1; r < NUM_ROWS; r++)
        document[r] = document[0] + (NUM_COLS*r);

    newDocument();

}

QColor TundraModel::getPalette(int index) const {
    if(color_mode == COLOR_4BIT) {
        if(index > -1 && index < NUM_COLORS)
            return palette[index];
        else
            return QColor(0,0,0);
    } else {
        if(index > -1) {
            // Parse out the colors
            int red = (index >> 16) & 0x000000FF;
            int green = (index >> 8) & 0x000000FF;
            int blue = index & 0x000000FF;

            return QColor(red, green, blue);
        } else
            return QColor(0,0,0);
    }
}

bool TundraModel::setCurrentFg(int color) {
    current_fg = color;
    emit fgColorChanged();
    return true;
}

bool TundraModel::setCurrentBg(int color) {
    current_bg = color;
    emit bgColorChanged();
    return true;
}

bool TundraModel::setCurrentPosition(int x, int y) {
    if(x > -1 && x < NUM_COLS && y > -1 && y < NUM_ROWS) {
        last_position.setX(current_position.x());
        last_position.setY(current_position.y());
        current_position.setX(x);
        current_position.setY(y);
        emit positionChanged();
        return true;
    } else
        return false;
}

bool TundraModel::setCurrentX(int x) {
    if(x > -1 && x < NUM_COLS) {
        last_position.setX(current_position.x());
        last_position.setY(current_position.y());
        current_position.setX(x);
        emit positionChanged();
        return true;
    } else
        return false;
}

bool TundraModel::setCurrentY(int y) {
    if(y > -1 && y < NUM_ROWS) {
        last_position.setY(current_position.y());
        last_position.setX(current_position.x());
        current_position.setY(y);
        emit positionChanged();
        return true;
    } else
        return false;
}

QPixmap* TundraModel::getCharacter(int index) {
    if(index > -1 && index < NUM_CHARACTERS) {
        return &characters[index];
    } else
        return NULL;
}

bool TundraModel::open24BitANSi(QFile& input, QByteArray* ba, TundraDoc** doc, int width, int height) {
    QDataStream* is = NULL;

    TundraDoc** current_document = document;
    if(doc != NULL) {
        current_document = doc;
        is = new QDataStream((*ba), IO_ReadOnly);
    } else {
        is = new QDataStream(&input);
        setColorMode(COLOR_24BIT);
    }

    int fg_color = 0;
    int bg_color = 0;
    Q_INT32 i=0;
    Q_INT32 j=0;

    while(!is->atEnd()) {
        Q_INT8 first;
        Q_INT8 second;
        (*is) >> first;
        if(first == (Q_INT8)1) {
            // Read the position
            (*is) >> i;
            (*is) >> j;

            if(i < height && j < width) {
                (*is) >> second;

                if(second > 1 && second <= 6) {
                    (*is) >> current_document[i][j].character;
                    if(second & 2) {
                        (*is) >> current_document[i][j].fg_color;
                        fg_color = current_document[i][j].fg_color;
                    } else
                        current_document[i][j].fg_color = fg_color;

                    if(second & 4) {
                        (*is) >> current_document[i][j].bg_color;
                        bg_color = current_document[i][j].bg_color;
                    } else
                        current_document[i][j].bg_color = bg_color;
                } else {
                    current_document[i][j].character = second;
                    current_document[i][j].fg_color = fg_color;
                    current_document[i][j].bg_color = bg_color;
                }
            } else {
                QMessageBox::critical(NULL, "Error opening file", "File format is invalid!");
                break;
            }
        } else {
            // Calculate the next position
            if(++j >= width) {
                i++;
                j = 0;
            }

            if(i < height && j < width) {
                if(first > 1 && first <= 6) {
                    (*is) >> current_document[i][j].character;
                    if(first & 2) {
                        (*is) >> current_document[i][j].fg_color;
                        fg_color = current_document[i][j].fg_color;
                    } else
                        current_document[i][j].fg_color = fg_color;

                    if(first & 4) {
                        (*is) >> current_document[i][j].bg_color;
                        bg_color = current_document[i][j].bg_color;
                    } else
                        current_document[i][j].bg_color = bg_color;

                } else {
                    current_document[i][j].character = first;
                    current_document[i][j].fg_color = fg_color;
                    current_document[i][j].bg_color = bg_color;
                }
            } else {
                QMessageBox::critical(NULL, "Error opening files", "File was too large!");
                break;
            }
        }
    }

    input.close();
    if(doc != current_document) {
        has_filename = true;
        save_filename = input.name();
        emit modified();
    }

    return true;
}

bool TundraModel::openANSi(QFile& input, QByteArray* ba, TundraDoc** doc, int width, int height) {

    TundraDoc** current_document = document;
    QTextStream* is = NULL;

    if(doc != NULL) {
        // We're copying into a block, not the document
        current_document = doc;
        is = new QTextStream((*ba), IO_ReadOnly);
    } else {
        is = new QTextStream(&input);
        setColorMode(COLOR_4BIT);
    }

    is->setEncoding(QTextStream::Latin1);
    QString s;
    int i = -1;
    int j = 0;
    int save_i = 0;
    int save_j = 0;

    unsigned char bg_color = 0;
    unsigned char fg_color = 7;
    unsigned char bold = 0;

    while ( !is->atEnd() ) {
        s = is->read();

        int m = 0;
        i++;
        j = 0;

        while((uint)m < s.length()) {
            bool is_bad = false;

            if(s[m] == '\r') {
                m++;
                continue;
            }
            if(s[m] == '\n') {
                i++;
                j=0;
                m++;
                continue;
            }
            if(s[m] == '\033') {
                // We have the start of a new sequence
                if((uint)++m >= s.length())
                    break;
                if(s[m] == '[') {
                    // The seqeunce is formatted correctly
                    m++;
                    QString number = "";
                    vector<int> numbers;
                    int num;

                    if((uint)m >= s.length())
                        break;

                    if(s[m] == '?' || s[m] == '=') {
                        // Mode change, eat characters until we find an I or an h
                        m++;
                        while((uint)m < s.length() && (s[m] != 'I' && s[m] != 'h')) {
                            // Eat characters
                            m++;
                        }

                        m++;
                    } else if(s[m] == 's') {
                        // Save cursor position
                        save_i = i;
                        save_j = j;
                        m++;
                    } else if(s[m] == 'u') {
                        // Restore cursor position
                        i = save_i;
                        j = save_j;
                        m++;
                    } else {
                        while((uint)m < s.length() && s[m] != 'm' && s[m] != 'A' && s[m] != 'B'
                                && s[m] != 'C' && s[m] != 'D' && s[m] != 'H'
                                && s[m] != 'f') {
                            while(s[m] >= '0' && s[m] <= '9') {
                                number = number + s[m];

                                if((uint)++m >= s.length())
                                    break;
                            }

                            if(s[m] == ';') {
                                if(number.length() > 0) {
                                    num = number.toInt();
                                    numbers.push_back(num);
                                }

                                number = "";
                                if((uint)++m >= s.length())
                                    break;
                            }
                            else if(s[m] != 'm' && s[m] != 'A' && s[m] != 'B'
                                    && s[m] != 'C' && s[m] != 'D' && s[m] != 'H'
                                    && s[m] != 'f') {
                                is_bad = true;
                                ++m;
                                break;
                            }

                        }

                        if(!is_bad) {
                            if(number.length() > 0) {
                                num = number.toInt();
                                numbers.push_back(num);
                            }

                            if((uint)m < s.length()) {

                                if(s[m] == 'H' || s[m] == 'f') {
                                    // Move to a particular row & column
                                    if(numbers.size() == 2) {
                                        if(numbers[0] > -1 && numbers[0] <= width &&
                                                numbers[1] > -1 && numbers[1] <= height) {
                                            i = numbers[0]-1;
                                            j = numbers[1]-1;
                                        }
                                    }

                                    m++;
                                } else {
                                    QChar s_m = s[m];
                                    bool has_fg = false;
                                    bool has_bg = false;
                                    bool has_bold = false;
                                    unsigned char current_fg = fg_color;
                                    unsigned char current_bg = bg_color;
                                    unsigned char current_bold = bold;
                                    uint n = 0;

                                    switch(s_m) {

                                    case 'm':
                                        // End of a color command
                                        for(n = 0; n < numbers.size(); n++) {
                                            if(numbers[n] >= 40 && numbers[n] <= 47) {
                                                current_bg = numbers[n] - 40;
                                                has_bg = true;
                                            } else if(numbers[n] >= 30 && numbers[n] <= 37) {
                                                current_fg = numbers[n] - 30;
                                                has_fg = true;
                                            } else if(numbers[n] == 1 && numbers.size() > 1) {
                                                current_bold = 1;
                                                has_bold = true;
                                            } else if(numbers[n] == 1 && numbers.size() == 1) {
                                                current_bold = 1;
                                                has_bold = true;
                                            } else if(numbers[n] == 0 && numbers.size() == 1) {
                                                current_bold = 0;
                                                has_bold = true;
                                                current_fg = 7;
                                                current_bg = 0;
                                                has_fg = true;
                                                has_bg = true;
                                            } else if(numbers[n] == 0 && numbers.size() > 1) {
                                                current_bold = 0;
                                                has_bold = true;

                                                if(!has_fg)
                                                    current_fg = 7;
                                                if(!has_bg)
                                                    current_bg = 0;

                                                has_fg = true;
                                                has_bg = true;
                                            }
                                        }

                                        if((current_bold == 1) && ((bold == 0) || has_fg) && (current_fg + 8 < NUM_COLORS))
                                            fg_color = current_fg + 8;
                                        else
                                            fg_color = current_fg;

                                        bg_color = current_bg;

                                        if(has_bold)
                                            bold = current_bold;

                                        break;
                                    case 'A':
                                        // Move cursor up X number[ lines
                                        if(numbers.size() == 1) {
                                            if(i-numbers[0] > -1) {
                                                i-=numbers[0];
                                            }
                                        } else if(numbers.size() == 0) {
                                            if(i > 0) {
                                                i-=1;
                                            }
                                        }
                                        break;

                                    case 'B':
                                        // Move cursor down X number lines
                                        if(numbers.size() == 1) {
                                            if(i+numbers[0] < width) {
                                                i+=numbers[0];
                                            }
                                        } else if(numbers.size() == 0) {
                                            if(i+1 < height) {
                                                i+=1;
                                            }
                                        }
                                        break;
                                    case 'C':
                                        // Move curor right X number lines
                                        if(numbers.size() == 1) {
                                            if(j+numbers[0] < width)
                                                j+=numbers[0];
                                            else {
                                                i = i + (int)((j + numbers[0])/width);
                                                j = (j + numbers[0])%width;
                                            }
                                        } else if(numbers.size() == 0) {
                                            if(j+1 < width) {
                                                j+=1;
                                            } else {
                                                i = i + (int)((j + 1)/width);
                                                j = (j + 1)%width;
                                            }
                                        }
                                        break;
                                    case 'D':
                                        // Move cursor left X number lines
                                        if(numbers.size() == 1) {
                                            if(j-numbers[0] > -1) {
                                                j-=numbers[0];
                                            }
                                        } else if(numbers.size() == 0) {
                                            if(j > 0) {
                                                j-=1;
                                            }
                                        }
                                        break;
                                    }
                                    m++;
                                }
                            }
                        }
                    }
                } else {
                    // The sequence was had a formatting error
                    if((uint)++m >= s.length())
                        break;
                }
            } else {
                if(j >= width) {
                    j = 0;
                    i++;
                }

                if(i > -1 && i < height && j > -1 && j < width) {

                    setMaxY(j);
                    QChar s_m = s[m];
                    current_document[i][j].character = (unsigned char)s_m;

                    current_document[i][j].fg_color = fg_color;
                    current_document[i][j].bg_color = bg_color;
                }
                m++;
                j++;
            }
        }
    }

    if(current_document != doc) {
        input.close();
        has_filename = true;
        save_filename = input.name();
        emit modified();
    } else {
        if(color_mode == COLOR_24BIT) {
            setColorMode(COLOR_24BIT, current_document, width, height);
        }
    }

    if(is != NULL)
        delete is;

    return true;
}

bool TundraModel::openDocument(QString filename) {

    QFile input(filename);
    if(input.open(IO_ReadOnly)) {
        bool is_24bit = true;

        int current_byte = input.getch();

        if(current_byte == 24) {
            for(unsigned int i = 0; i < strlen(header_24bit); i++) {
                if(input.getch() != (int)header_24bit[i]) {
                    is_24bit = false;
                    break;
                }
            }
        } else
            is_24bit = false;

        if(!is_24bit) {
            input.reset();
            return openANSi(input);
        } else {
            return open24BitANSi(input);
        }
    }
    else {
        return false;
    }
}

void TundraModel::newDocument() {
    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            document[i][j].character = 0;
            document[i][j].fg_color = 0;
            document[i][j].bg_color = 0;
        }
    }

    current_position.setX(0);
    current_position.setY(0);

    max_y = NUM_ROWS-1;
    max_x = NUM_COLS-1;

    has_filename = false;
    save_filename = QString("");
}

bool TundraModel::saveDocument(QString filename, TundraSaveStrategy* strategy) {
    current_save_strategy = strategy;

    if(!has_filename || filename != save_filename) {
        has_filename = true;
        save_filename = filename;
    }

    // Just in case current_save_strategy was NULL and we already had a filename
    if(strategy == NULL) {
        if(filename.endsWith(".asc"))
            strategy = new TundraSaveAsciiStrategy();
        else if(color_mode == COLOR_24BIT)
            strategy = new TundraSave24BitStrategy();
        else
            strategy = new TundraSaveAnsiStrategy();
    }

    return strategy->saveFile(filename, document, max_y);
}

bool TundraModel::saveDocument() {
    if(saveDocument(save_filename, current_save_strategy))
        return true;
    else {
        return false;
    }
}

void TundraModel::setColorMode(unsigned char mode, TundraDoc** doc, int width, int height) {
    TundraDoc** current_document = document;
    if(doc != NULL) {
        // For setting the color of something we're pasting
        current_document = doc;
    }

    last_color_mode = color_mode;
    color_mode = mode;
    // Convert current fg color and current bg color to 24-bit index
    if((last_color_mode == COLOR_4BIT && mode == COLOR_24BIT) || doc != NULL) {
        QColor fg_color;
        QColor bg_color;

        int new_fg;
        int new_bg;

        if(doc == NULL) {
            fg_color = palette[current_fg];
            bg_color = palette[current_bg];
            new_fg = packColor(fg_color.red(), fg_color.green(), fg_color.blue());
            new_bg = packColor(bg_color.red(), bg_color.green(), bg_color.blue());

            current_fg = new_fg;
            current_bg = new_bg;
        }

        // Run through model and convert everything to 24-bit index
        for(int i = 0; i < height; i++) {
            for(int j = 0; j < width; j++) {
                if(current_document[i][j].fg_color >= 0 && current_document[i][j].fg_color < 16) {
                    fg_color = palette[current_document[i][j].fg_color];
                    new_fg = packColor(fg_color.red(), fg_color.green(), fg_color.blue());
                    current_document[i][j].fg_color = new_fg;
                }
                if(current_document[i][j].bg_color >= 0 && current_document[i][j].bg_color < 16) {
                    bg_color = palette[current_document[i][j].bg_color];
                    new_bg = packColor(bg_color.red(), bg_color.green(), bg_color.blue());
                    current_document[i][j].bg_color = new_bg;
                }
            }
        }

    } else if(last_color_mode == COLOR_24BIT && mode == COLOR_4BIT) {
        // Attempt to convert colors back to 4-bit
        current_fg = 7;
        current_bg = 0;

        for(int i = 0; i < height; i++) {
            for(int j = 0; j < width; j++) {
                int smallest_difference1 = 256;
                int smallest_difference2 = 256;
                int smallest_difference3 = 256;

                int smallest_index = 0;

                int red = (current_document[i][j].fg_color >> 16) & 0x000000FF;
                int green = (current_document[i][j].fg_color >> 8) & 0x000000FF;
                int blue = current_document[i][j].fg_color & 0x000000FF;

                QValueVector<int> color_factor;
                color_factor.push_back(red);
                color_factor.push_back(green);
                color_factor.push_back(blue);

                qHeapSort(color_factor);

                QValueVector<int> factor_colors(3);

                bool red_used, blue_used, green_used;
                red_used = green_used = blue_used = false;

                for(int m = 0; m < 3; m++) {
                    if(!red_used && color_factor[m] == red) {
                            factor_colors[m] = COLOR_RED;
                            red_used = true;
                    }
                    else if(!blue_used && color_factor[m] == blue) {
                            factor_colors[m] = COLOR_BLUE;
                            blue_used = true;
                    }
                    else if(!green_used) {
                            factor_colors[m] = COLOR_GREEN;
                            green_used = true;
                    }
                }

                for(int n = 0; n < NUM_COLORS; n++) {

                    if((red == palette[n].red()) && (blue == palette[n].blue()) && (green == palette[n].green())) {
                        smallest_index = n;
                        break;    
                    }

                    QValueVector<int> palette_colors;
                    palette_colors.push_back(palette[n].red());
                    palette_colors.push_back(palette[n].green());
                    palette_colors.push_back(palette[n].blue());
                    
                    int difference1 = abs(color_factor[0] - palette_colors[factor_colors[0]]);
                    int difference2 = abs(color_factor[1] - palette_colors[factor_colors[1]]);
                    int difference3 = abs(color_factor[2] - palette_colors[factor_colors[2]]);

                    if(difference1 <= smallest_difference1) {
                        if(difference2 < smallest_difference2) {
                            smallest_difference1 = difference1;
                            smallest_difference2 = difference2;
                            smallest_difference3 = difference3;
                            smallest_index = n;
                        }
                        else if(difference2 == smallest_difference2) {
                            if(difference3 < smallest_difference3) {
                                smallest_difference1 = difference1;
                                smallest_difference2 = difference2;
                                smallest_difference3 = difference3;
                                smallest_index = n;
                            }
                        }
                    }

                    if(smallest_difference1 == 0 && smallest_difference2 == 0 && smallest_difference3 == 0)
                        break;
                }

                current_document[i][j].fg_color = smallest_index;

                // Convert background color
                smallest_difference1 = 256;
                smallest_difference2 = 256;
                smallest_difference3 = 256;

                smallest_index = 0;

                red = (current_document[i][j].bg_color >> 16) & 0x000000FF;
                green = (current_document[i][j].bg_color >> 8) & 0x000000FF;
                blue = current_document[i][j].bg_color & 0x000000FF;

                color_factor.clear();
                color_factor.push_back(red);
                color_factor.push_back(green);
                color_factor.push_back(blue);

                qHeapSort(color_factor);

                factor_colors.clear();
                factor_colors.resize(3);

                red_used = green_used = blue_used = false;

                for(int m = 0; m < 3; m++) {
                    if(!red_used && color_factor[m] == red) {
                            factor_colors[m] = COLOR_RED;
                            red_used = true;
                    }
                    else if(!blue_used && color_factor[m] == blue) {
                            factor_colors[m] = COLOR_BLUE;
                            blue_used = true;
                    }
                    else if(!green_used) {
                            factor_colors[m] = COLOR_GREEN;
                            green_used = true;
                    }
                }

                for(int n = 0; n < 8; n++) {

                    if((red == palette[n].red()) && (blue == palette[n].blue()) && (green == palette[n].green())) {
                        smallest_index = n;
                        break;    
                    }

                    QValueVector<int> palette_colors;
                    palette_colors.push_back(palette[n].red());
                    palette_colors.push_back(palette[n].green());
                    palette_colors.push_back(palette[n].blue());
                    
                    int difference1 = abs(color_factor[0] - palette_colors[factor_colors[0]]);
                    int difference2 = abs(color_factor[1] - palette_colors[factor_colors[1]]);
                    int difference3 = abs(color_factor[2] - palette_colors[factor_colors[2]]);

                    if(difference1 <= smallest_difference1) {
                        if(difference2 < smallest_difference2) {
                            smallest_difference1 = difference1;
                            smallest_difference2 = difference2;
                            smallest_difference3 = difference3;
                            smallest_index = n;
                        }
                        else if(difference2 == smallest_difference2) {
                            if(difference3 < smallest_difference3) {
                                smallest_difference1 = difference1;
                                smallest_difference2 = difference2;
                                smallest_difference3 = difference3;
                                smallest_index = n;
                            }
                        }
                    }

                    if(smallest_difference1 == 0 && smallest_difference2 == 0 && smallest_difference3 == 0)
                        break;
                }

                current_document[i][j].bg_color = smallest_index;

            }
        }

    }

    if(doc == NULL) {
        emit colorModeChanged();
        emit modified();
    }

    has_filename = false;
    save_filename = QString("");
}

bool TundraModel::exportUnitTestData(QString filename) {
    QDataStream* os = NULL;
    QFile output(filename);

    bool return_val = false;

    if(!filename.isNull()) {
        output.setName(filename);
        if(output.open(IO_WriteOnly | IO_Truncate)) {
            os = new QDataStream(&output);

            for(int i = 0; i < NUM_ROWS; i++) {
                for(int j = 0; j < NUM_COLS; j++) {
                    (*os) << (Q_UINT8)document[i][j].character;
                    (*os) << (Q_UINT32)document[i][j].fg_color;
                    (*os) << (Q_UINT32)document[i][j].bg_color;
                }
            }

            return_val = true;
        }
    }

    return return_val;
}

bool TundraModel::importUnitTestData(QString filename, unsigned int color_mode) {
    bool return_val = false;

    newDocument();
    QFile input(filename);
    if(input.open(IO_ReadOnly)) {

        QDataStream* is = NULL;

        is = new QDataStream(&input);
        setColorMode(color_mode);

        for(int i = 0; i < NUM_ROWS; i++) {
            for(int j = 0; j < NUM_COLS; j++) {
                (*is) >> document[i][j].character;
                (*is) >> document[i][j].fg_color;
                (*is) >> document[i][j].bg_color;
            }

            return_val = true;
        }
    }
    return return_val;
}

bool operator==( const TundraModel &a, const TundraModel &b) {
    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            TundraDoc* a_item = a.getItem(j, i);
            TundraDoc* b_item = b.getItem(j, i);
            if(!(a_item->character == 0 && b_item->character == 0) && ((a_item->character != b_item->character) ||
                (a_item->fg_color != b_item->fg_color) ||
                (a_item->bg_color != b_item->bg_color))) {
                qDebug("### Found document mismatch at %d, %d in %s!\n", j, i, b.save_filename.ascii());
                qDebug("### item a:\ncharacter: %d\nfg_color: %d\n bg_color: %d\n", a_item->character,
                        a_item->fg_color, a_item->bg_color);
                qDebug("### item b:\ncharacter: %d\nfg_color: %d\n bg_color: %d\n", b_item->character,
                        b_item->fg_color, b_item->bg_color);
                return false;
            }
        }
    }
    return true;
}
