/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundramovecommand.h"

TundraMoveCommand::TundraMoveCommand(TundraModel* m, QPoint& tl, QPoint& br) : is_remote(false)
{
    model = m;

    source_tl = tl;
    source_br = br;

    width = source_br.x()-source_tl.x();
    height = source_br.y()-source_tl.y();

    source_block = new TundraDoc*[height];
    source_block[0] = new TundraDoc[width*height];

    for(int r = 1; r < height; r++)
        source_block[r] = source_block[0] + (width*r);

    dest_block = new TundraDoc*[height];
    dest_block[0] = new TundraDoc[width*height];

    for(int r = 1; r < height; r++)
        dest_block[r] = dest_block[0] + (width*r);

    TundraDoc* current;

    // Copy the old block
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            current = model->getItem(x+source_tl.x(), y+source_tl.y());
            source_block[y][x].character = current->character;
            source_block[y][x].bg_color = current->bg_color;
            source_block[y][x].fg_color = current->fg_color;
        }
    }

}

TundraMoveCommand::~TundraMoveCommand(void)
{
    delete [] source_block[0];
    delete [] source_block;

    delete [] dest_block[0];
    delete [] dest_block;
}

void TundraMoveCommand::execute() {
    TundraDoc* current;

    // Cut out the old block
    for(int y = source_tl.y(); y < source_br.y(); y++) {
        for(int x = source_tl.x(); x < source_br.x(); x++) {
            current = model->getItem(x, y);

            current->character = 0;
            current->fg_color = 0;
            current->bg_color = 0;
        }
    }

    // Paste the new block
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            if(((x+dest_tl.x()) <= model->getMaxX()) && ((x+dest_tl.x()) >= 0) && ((y+dest_tl.y()) >= 0) && ((y+dest_tl.y()) <= model->getMaxY())) {
                current = model->getItem(x+dest_tl.x(), y+dest_tl.y());
                current->character = source_block[y][x].character;
                current->bg_color = source_block[y][x].bg_color;
                current->fg_color = source_block[y][x].fg_color;
            }
        }
    }

    model->setModified();
}

void TundraMoveCommand::unexecute() {

    TundraDoc* current;

    // Copy the old block back that was overwritten by the paste
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            if(((x+dest_tl.x()) <= model->getMaxX()) && ((x+dest_tl.x()) >= 0) && ((y+dest_tl.y()) >= 0) && ((y+dest_tl.y()) <= model->getMaxY())) {
                current = model->getItem(x+dest_tl.x(), y+dest_tl.y());
                current->character = dest_block[y][x].character;
                current->bg_color = dest_block[y][x].bg_color;
                current->fg_color = dest_block[y][x].fg_color;
            }
        }
    }

    // Copy the old source block back that was cut out
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            current = model->getItem(x+source_tl.x(), y+source_tl.y());

            current->character = source_block[y][x].character;
            current->fg_color = source_block[y][x].fg_color;
            current->bg_color = source_block[y][x].bg_color;
        }
    }

    model->setModified();
}

void TundraMoveCommand::setDestination(QPoint& tl) {
    dest_tl = tl;

    TundraDoc* current;

    // Save the old block
    for(int y = 0; y < height; y++) {
        for(int x = 0; x < width; x++) {
            if(((x+dest_tl.x()) <= model->getMaxX()) && ((x+dest_tl.x()) >= 0) && ((y+dest_tl.y() >= 0)) && ((y+dest_tl.y()) <= model->getMaxY())) {
                current = model->getItem(x+dest_tl.x(), y+dest_tl.y());
                dest_block[y][x].character = current->character;
                dest_block[y][x].bg_color = current->bg_color;
                dest_block[y][x].fg_color = current->fg_color;
            }
        }
    }
}
