/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAMOVECOMMAND_H
#define TUNDRAMOVECOMMAND_H

#include <qpoint.h>

#include "tundracommand.h"
#include "tundramodel.h"
#include "tundracommandvisitor.h"

//! Move a block of characters
/*!
    Move is a two part command. The first part starts the move, the second part actually
    places the block. This is done using setDestination(). Note that this is not a traditional
    block command, but its own command.
*/
class TundraMoveCommand :
            public TundraCommand
{
public:
    //! Constructor
    /*!
        \param m A pointer to the TundraModel
        \param tl A QPoint representing the top left corner of the block to move
        \param br A QPoint representing the bottom right corner of the block to move
    */
    TundraMoveCommand(TundraModel* m, QPoint& tl, QPoint& br);

    //! Destructor
    ~TundraMoveCommand(void);

    //! Execture the move
    void execute();

    //! Undo the move
    void unexecute();

    //! Accept a visitor
    /*!
        \param v A pointer to the visitor
    */
    void accept(TundraCommandVisitor* v) { v->visitMoveCommand(this); }

    //! Set the destination of the move
    /*!
        \param tl A QPoint representing the position of the new top left corner for the block
    */
    void setDestination(QPoint& tl);

    //! Specify that this is a network command
    void setRemote() { is_remote = true; }

    //! A QPoint representing the original top left corner
    QPoint source_tl;

    //! A QPoint representing the original bottom right corner
    QPoint source_br;

    //! A QPoint representing the position of the new top left corner for the block
    QPoint dest_tl;
private:
    TundraModel* model;
    TundraDoc** source_block;
    TundraDoc** dest_block;
    int width;
    int height;
    bool is_remote;
};

#endif
