/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRANETBLOCKSTRATEGYVISITOR_H
#define TUNDRANETBLOCKSTRATEGYVISITOR_H

#include "tundrablockstrategyvisitor.h"
#include "tundraerasestrategy.h"
#include "tundrafillattributestrategy.h"
#include "tundrafillbgstrategy.h"
#include "tundrafillbothstrategy.h"
#include "tundrafillcharstrategy.h"
#include "tundrafillfgstrategy.h"
#include "tundraflipxstrategy.h"
#include "tundraflipystrategy.h"
#include "tundrapastestrategy.h"

//! Network block strategy visitor
/*!
    The network block strategy visitor constructs network commands
    representing the various block strategies
*/
class TundraNetBlockStrategyVisitor : public TundraBlockStrategyVisitor
{
public:
    //! Constructor
    TundraNetBlockStrategyVisitor();

    //! Destructor
    virtual ~TundraNetBlockStrategyVisitor();

    //! Visit the erase strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitEraseStrategy(TundraEraseStrategy* s);

    //! Visit the fill attribute strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFillAttributeStrategy(TundraFillAttributeStrategy* s);

    //! Visit the fill background strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFillBgStrategy(TundraFillBgStrategy* s);

    //! Visit the fill both character and attribute strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFillBothStrategy(TundraFillBothStrategy* s);

    //! Visit the fill character strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFillCharStrategy(TundraFillCharStrategy* s);

    //! Visit the fill foreground strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFillFgStrategy(TundraFillFgStrategy* s);

    //! Visit the flip x strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFlipXStrategy(TundraFlipXStrategy* s);

    //! Visit the flip y strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitFlipYStrategy(TundraFlipYStrategy* s);

    //! Visit the paste strategy
    /*!
        \param s A pointer to the strategy
    */
    void visitPasteStrategy(TundraPasteStrategy* s);

    //! Get the command buffer
    char* getCommandBuffer() const { return command; }

    //! Get the command length
    int getCommandLength() const { return command_length; }
private:
    char* command;
    int command_length;
};

#endif
