/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundranetcommandprocessor.h"

TundraNetCommandProcessor::TundraNetCommandProcessor(TundraModel* m) : TundraCommandProcessor(m)
{
}

TundraNetCommandProcessor::~TundraNetCommandProcessor(void)
{
}

Q_ULONG TundraNetCommandProcessor::parseNetCommand(char* data, Q_ULONG length) {
    int i = 0;

    // Check the command
    switch(data[i]) {
    case 0:
        // Load ANSi command
        // First get the character count, followed by a pipe
        if((length >= 7) && (data[5] == '|')) {
            int character_count = packInteger(data+1);
            char color_mode = data[6];

            // We use 17 bytes to pack the x,y,character,bg_color,fg_color
            if(length >= (unsigned int)((character_count*17) + 7)) {
                model->sendNewDocumentWarning();
                // Don't set the color mode until after the user has been prompted to save
                model->setColorMode((unsigned char)color_mode);
                model->newDocument();
                model->sendNewDocumentNotify();

                for(int j = 0; j < (character_count*17); j+=17) {
                    int x, y;
                    x = packInteger(data+j+7);
                    y = packInteger(data+j+4+7);

                    unsigned char character;
                    int bg_color, fg_color;

                    character = data[j+8+7];
                    bg_color = packInteger(data+j+9+7);
                    fg_color = packInteger(data+j+13+7);

                    TundraDoc* loc = model->getItem(x, y);
                    loc->fg_color = fg_color;
                    loc->bg_color = bg_color;
                    loc->character = character;
                }
                model->setModified();

                i += (character_count*17) + 7;
            }
        }
        break;
    case 3:
        if(length >= 1) {
            undo();
            i += 1;
        }
        break;
    case 4:
        if(length >= 1) {
            redo();
            i += 1;
        }
        break;
    case 5:
        if(length >= 18) {
            // Character command
            char character;
            int bg_color, fg_color;
            int x = packInteger(data+1);
            int y = packInteger(data+5);
            character = data[9];
            bg_color = packInteger(data+10);
            fg_color = packInteger(data+14);

            TundraCharacterCommand* command = newCharacterCommand(x, y, character, bg_color, fg_color);
            command->setRemote();
            process(command);
            i += 18;
        }
        break;
    case 6:
        // User logged in message
        if(length >= 2) {
            char name_length = data[1];
            if(name_length > 0 && length >= ((uint)name_length+2)) {
                QString name = QString(data+2);
                name.truncate((uint)name_length);
                emit userLoggedIn(name);
                i += (name_length+2);
            }
        }
        break;
    case 7:
        // User logged out message
        if(length >= 2) {
            char name_length = data[1];
            if(name_length > 0 && length >= ((uint)name_length+2)) {
                QString name = QString(data+2);
                name.truncate((uint)name_length);
                emit userLoggedOut(name);
                i += (name_length+2);
            }
        }
        break;
    case 8:
        // Chat message
        if(length >= 5) {
            int name_length = packInteger(data+1);
            if(name_length > 0 && length >= ((uint)name_length + 9)) {
                int message_length = packInteger(data+name_length+5);
                if(message_length > 0 && length >= ((uint)name_length+5+(uint)message_length+4)) {
                    QString name = QString(data+5);
                    name.truncate((uint)name_length);

                    QString message = QString(data+5+name_length+4);
                    message.truncate((uint)message_length);
                    emit userMessage(name, message);
                    i += (name_length+5+message_length+4);
                }
            }
        }
        break;
    case 9:
        // Delete command
        if(length >= 9) {
            int x = packInteger(data+1);
            int y = packInteger(data+5);

            TundraDeleteCommand* command = newDeleteCommand(x, y);
            command->setRemote();
            process(command);
            i += 9;
        }

        break;
    case 10:
        if(length >= 18) {
            // Insert command
            char character;
            int bg_color, fg_color;
            int x = packInteger(data+1);
            int y = packInteger(data+5);
            character = data[9];
            bg_color = packInteger(data+10);
            fg_color = packInteger(data+14);

            TundraInsertCommand* command = newInsertCommand(x, y, character, bg_color, fg_color);
            command->setRemote();
            process(command);
            i += 18;
        }
        break;
    case 11:
        // Block command
        if(length >= 18) {
            int x1, y1, x2, y2;
            x1 = packInteger(data+1);
            y1 = packInteger(data+5);
            x2 = packInteger(data+9);
            y2 = packInteger(data+13);

            QPoint tl(x1, y1);
            QPoint br(x2, y2);

            switch(data[17]) {
            case 0: {
                    // Erase Strategy
                    TundraEraseStrategy* s = new TundraEraseStrategy();
                    TundraBlockCommand* command = newBlockCommand(tl, br, s);
                    command->setRemote();
                    process(command);
                    i += 18;
                }
                break;
            case 1: {
                    // Fill Attribute Strategy
                    if(length >= 26) {
                        int fg_color = packInteger(data+18);
                        int bg_color = packInteger(data+22);
                        TundraFillAttributeStrategy* s = new TundraFillAttributeStrategy(fg_color, bg_color);
                        TundraBlockCommand* command = newBlockCommand(tl, br, s);
                        command->setRemote();
                        process(command);
                        i += 26;
                    }
                }
                break;

            case 2: {
                    // Fill Background Strategy
                    if(length >= 22) {
                        int bg_color = packInteger(data+18);
                        TundraFillBgStrategy* s = new TundraFillBgStrategy(bg_color);
                        TundraBlockCommand* command = newBlockCommand(tl, br, s);
                        command->setRemote();
                        process(command);
                        i += 22;
                    }
                }
                break;

            case 3: {
                    // Fill Both Strategy
                    if(length >= 30) {
                        int character = packInteger(data+18);
                        int fg_color = packInteger(data+22);
                        int bg_color = packInteger(data+26);
                        TundraFillBothStrategy* s = new TundraFillBothStrategy(character, fg_color, bg_color);
                        TundraBlockCommand* command = newBlockCommand(tl, br, s);
                        command->setRemote();
                        process(command);
                        i += 30;
                    }
                }
                break;

            case 4: {
                    // Fill Foreground Strategy
                    if(length >= 22) {
                        int fg_color = packInteger(data+18);
                        TundraFillFgStrategy* s = new TundraFillFgStrategy(fg_color);
                        TundraBlockCommand* command = newBlockCommand(tl, br, s);
                        command->setRemote();
                        process(command);
                        i += 22;
                    }
                }
                break;

            case 5: {
                    // Paste Strategy
                    if(length >= 26) {
                        int source_width = packInteger(data+18);
                        int source_height = packInteger(data+22);

                        if(length >= (unsigned)(26+(source_width*source_height*9))) {
                            TundraDoc** doc = new TundraDoc*[source_height];
                            doc[0] = new TundraDoc[source_height*source_width];

                            for(int m = 1; m < source_height; m++) {
                                doc[m] = doc[0]+(source_width*m);
                            }

                            for(int m = 0; m < (source_width*source_height); m++) {
                                int x = m % source_width;
                                int y = m / source_width;

                                int loc = 26+(m*9);

                                doc[y][x].character = data[loc];

                                loc = 26 + (m*9)+1;

                                doc[y][x].fg_color = packInteger(data+loc);

                                loc = 26 + (m*9)+5;

                                doc[y][x].bg_color = packInteger(data+loc);
                            }

                            TundraPasteStrategy* s = new TundraPasteStrategy(doc, source_width, source_height);
                            TundraBlockCommand* command = newBlockCommand(tl, br, s);
                            command->setRemote();
                            process(command);

                            delete [] doc[0];
                            delete [] doc;

                            i += 26+(source_width*source_height*9);

                        }

                    }
                }
                break;
            case 6: {
                    // Flip X Strategy
                    TundraFlipXStrategy* s = new TundraFlipXStrategy();
                    TundraBlockCommand* command = newBlockCommand(tl, br, s);
                    command->setRemote();
                    process(command);
                    i += 18;
                }
                break;
            case 7: {
                    // Flip Y Strategy
                    TundraFlipYStrategy* s = new TundraFlipYStrategy();
                    TundraBlockCommand* command = newBlockCommand(tl, br, s);
                    command->setRemote();
                    process(command);
                    i += 18;
                }
                break;
            case 8:
                // Fill Character Strategy
                if(length >= 30) {
                    int character = packInteger(data+18);
                    int fg_color = packInteger(data+22);
                    int bg_color = packInteger(data+26);
                    TundraFillCharStrategy* s = new TundraFillCharStrategy(character, fg_color, bg_color);
                    TundraBlockCommand* command = newBlockCommand(tl, br, s);
                    command->setRemote();
                    process(command);
                    i += 30;
                }
            }

        }

        break;
    case 12:
        if(length >= 25) {
            int source_x1 = packInteger(data+1);
            int source_y1 = packInteger(data+5);
            int source_x2 = packInteger(data+9);
            int source_y2 = packInteger(data+13);
            int dest_x1 = packInteger(data+17);
            int dest_y1 = packInteger(data+21);

            QPoint source_tl(source_x1, source_y1);
            QPoint source_br(source_x2, source_y2);
            QPoint dest_tl(dest_x1, dest_y1);

            TundraMoveCommand* command = newMoveCommand(source_tl, source_br);
            command->setDestination(dest_tl);

            command->setRemote();
            process(command);

            i += 25;
        }
        break;
    case 13:
        if(length >= 2) {
            unsigned char color_mode = (unsigned char)data[1];
            model->setColorMode(color_mode);
            i += 2;
        }
        break;
    default:
        QMessageBox::information(NULL, "Error", "Received unknown command");
        break;
    }

    return i;
}


