/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundranetcommandvisitor.h"
#include "tundranetblockstrategyvisitor.h"

TundraNetCommandVisitor::TundraNetCommandVisitor() : client_socket(NULL), server_socket(NULL)
{
}

TundraNetCommandVisitor::~TundraNetCommandVisitor(void)
{
}

void TundraNetCommandVisitor::visitCharacterCommand(TundraCharacterCommand* c) {
    if((client_socket != NULL && client_socket->isLoggedIn()) || server_socket != NULL) {
        int x = c->my_x;
        int y = c->my_y;

        int command_length = 18;
        char* command = new char[command_length];
        command[0] = 5;

        // X portion of command
        unpackInteger(x, command+1);

        // Y portion of command
        unpackInteger(y, command+5);

        command[9] = c->new_doc.character;

        unpackInteger(c->new_doc.bg_color, command+10);

        unpackInteger(c->new_doc.fg_color, command+14);

        if(client_socket != NULL) {
            client_socket->writeBlock(command, command_length);
        }
        else if(server_socket != NULL)
            server_socket->writeCommand(command, command_length);

        delete command;
    }
}

void TundraNetCommandVisitor::visitDeleteCommand(TundraDeleteCommand* c) {
    int x = c->my_x;
    int y = c->my_y;

    int command_length = 9;
    char* command = new char[command_length];
    command[0] = 9;

    // X portion of command
    unpackInteger(x, command+1);

    // Y portion of command
    unpackInteger(y, command+5);

    if(client_socket != NULL) {
        client_socket->writeBlock(command, command_length);
    }
    else if(server_socket != NULL)
        server_socket->writeCommand(command, command_length);

    delete command;

}

void TundraNetCommandVisitor::visitInsertCommand(TundraInsertCommand* c) {
    if((client_socket != NULL && client_socket->isLoggedIn()) || server_socket != NULL) {
        int x = c->my_x;
        int y = c->my_y;

        int command_length = 18;
        char* command = new char[command_length];
        command[0] = 10;

        // X portion of command
        unpackInteger(x, command+1);

        // Y portion of command
        unpackInteger(y, command+5);

        command[9] = c->new_doc.character;

        unpackInteger(c->new_doc.bg_color, command+10);

        unpackInteger(c->new_doc.fg_color, command+14);

        if(client_socket != NULL) {
            client_socket->writeBlock(command, command_length);
        }
        else if(server_socket != NULL)
            server_socket->writeCommand(command, command_length);

        delete command;
    }
}

void TundraNetCommandVisitor::visitBlockCommand(TundraBlockCommand* c) {
    TundraNetBlockStrategyVisitor* visitor = new TundraNetBlockStrategyVisitor();
    c->strategy->accept(visitor);

    char* command = visitor->getCommandBuffer();

    command[0] = 11;

    // x1, y1, x2, y2
    unpackInteger(c->top_left.x(), command+1);
    unpackInteger(c->top_left.y(), command+5);
    unpackInteger(c->bottom_right.x(), command+9);
    unpackInteger(c->bottom_right.y(), command+13);

    int command_length = visitor->getCommandLength();

    if(client_socket != NULL)
        client_socket->writeBlock(command, command_length);
    else if(server_socket != NULL)
        server_socket->writeCommand(command, command_length);
}

void TundraNetCommandVisitor::visitMoveCommand(TundraMoveCommand* c) {
    int command_length = 25;
    char* command = new char[command_length];
    command[0] = 12;

    unpackInteger(c->source_tl.x(), command+1);
    unpackInteger(c->source_tl.y(), command+5);
    unpackInteger(c->source_br.x(), command+9);
    unpackInteger(c->source_br.y(), command+13);
    unpackInteger(c->dest_tl.x(), command+17);
    unpackInteger(c->dest_tl.y(), command+21);

    if(client_socket != NULL)
        client_socket->writeBlock(command, command_length);
    else if(server_socket != NULL)
        server_socket->writeCommand(command, command_length);
}

void TundraNetCommandVisitor::undo() {
    char command = 3;

    if(client_socket != NULL)
        client_socket->writeBlock(&command, 1);
    else if(server_socket != NULL)
        server_socket->writeCommand(&command, 1);
}

void TundraNetCommandVisitor::redo() {
    char command = 4;

    if(client_socket != NULL)
        client_socket->writeBlock(&command, 1);
    else if(server_socket != NULL)
        server_socket->writeCommand(&command, 1);
}

