/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRANETCOMMANDVISITOR_H
#define TUNDRANETCOMMANDVISITOR_H

#include "tundracommandvisitor.h"
#include "tundraclientsocket.h"
#include "tundraserversocket.h"
#include "tundracharactercommand.h"
#include "tundradeletecommand.h"
#include "tundrainsertcommand.h"
#include "tundrablockcommand.h"
#include "tundramovecommand.h"
#include "utils.h"

#include <qobject.h>

//! Network command visior
/*!
    This class is responsible for constructing actual network command messages by
    visiting commands that have been processed by the command processor, and then sending
    the commands over the socket.
*/
class TundraNetCommandVisitor : public QObject, public TundraCommandVisitor
{
    Q_OBJECT

public:
    //! Constructor
    TundraNetCommandVisitor();

    //! Destructor
    virtual ~TundraNetCommandVisitor(void);

    //! Set the client socket
    /*!
        \param s A pointer to the client socket
    */
    void setClient(TundraClientSocket* s) { client_socket = s; }

    //! Set the server socket
    /*!
        \param s A pointer to the server socket
    */
    void setServer(TundraServerSocket* s) { server_socket = s; }

    //! Notify the class that the client socket was disconnected
    void setClientDisconnected() { client_socket = NULL; }

    //! Notify the class that the server socket was disconnected
    void setServerDisconnected() { server_socket = NULL; }

    //! Visit the character command and construct network message
    /*!
        \param c A pointer to the character command
    */
    void visitCharacterCommand(TundraCharacterCommand* c);

    //! Visit the delete command and construct network message
    /*!
        \param c A pointer to the delete command
    */
    void visitDeleteCommand(TundraDeleteCommand* c);

    //! Visit the insert command and construct network message
    /*!
        \param c A pointer to the character command
    */
    void visitInsertCommand(TundraInsertCommand* c);

    //! Visit a block command and construct network message
    /*!
        \param c A pointer to the block command
    */
    void visitBlockCommand(TundraBlockCommand* c);

    //! Visit the move command and construct network message
    /*!
        \param c A pointer to the move command
    */
    void visitMoveCommand(TundraMoveCommand* c);
    
    //! Send an undo command
    void undo();

    //! Send a redo command
    void redo();
public slots:
    //! Disconnect any open sockets
    void setDisconnected() { client_socket = NULL; server_socket = NULL; }

private:
    TundraClientSocket* client_socket;
    TundraServerSocket* server_socket;
};

#endif
