/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAPASTESTRATEGY_H
#define TUNDRAPASTESTRATEGY_H

#include <qpoint.h>

#include "tundrablockstrategy.h"
#include "tundrablockstrategyvisitor.h"

//! The paste strategy
/*!
    Pastes a block of data (either from the clipboard or a quick copy)
*/
class TundraPasteStrategy :
            public TundraBlockStrategy
{
public:
    //! Constructor
    /*!
        \param doc A pointer to the block of characters to paste
        \param width The width of the block to paste
        \param height The height of the block to paste
    */
    TundraPasteStrategy(TundraDoc** doc, int width, int height);

    //! Destructor
    virtual ~TundraPasteStrategy(void);

    //! Execute the paste
    /*!
        \param m A pointer to the model
        \param tl A QPoint representing the top left corner of the destination
        \param br A QPoint representing the bottom right corner of the destination
    */
    void execute(TundraModel* m, QPoint& tl, QPoint&);

    //! Accept a visitor
    /*!
        \param v A pointer to the block strategy visitor
    */
    void accept(TundraBlockStrategyVisitor* v) { v->visitPasteStrategy(this); }

    //! The width of the source block
    int source_width;

    //! The height of the source block
    int source_height;

    //! A pointer to the block of characters to paste
    TundraDoc** source_doc;
};

#endif
