/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrapreview.h"

TundraPreview::TundraPreview(TundraModel* m, TundraCommandProcessor* cp, QWidget* parent, const char* name) : QMainWindow(parent, name), model(m), command_processor(cp) {

    resize(175, 400);
    setCaption("Preview Window");

    canvas_group = new TundraCanvasGroup(model, command_processor, 160, 400, this);
    canvas_group->setFrameShape(QFrame::WinPanel);
    canvas_group->setFrameShadow(QFrame::Sunken);
    canvas_group->setMinimumSize(180,400);
    canvas_group->setFixedWidth(180);

    QToolBar* picker_bar = new QToolBar(this);

    setDockEnabled(picker_bar, Right, true);
    setDockEnabled(picker_bar, Left, true);
    setDockEnabled(picker_bar, Bottom, true);
    setDockEnabled(picker_bar, TornOff, true);

    selection_group = new QButtonGroup(2, Qt::Vertical, picker_bar);
    selection_group->setTitle("Resolution");
    vga_button = new QRadioButton("VGA", selection_group);
    svga_button = new QRadioButton("SVGA", selection_group);

    selection_group->setButton(0);

    setCentralWidget(canvas_group);

    QAccel* a = new QAccel(this);
    a->connectItem(a->insertItem(Key_Z+CTRL), this, SLOT(undo()));
    a->connectItem(a->insertItem(Key_Y+CTRL), this, SLOT(redo()));

    QObject::connect(svga_button, SIGNAL(clicked()), this, SLOT(svgaClicked()));
    QObject::connect(vga_button, SIGNAL(clicked()), this, SLOT(vgaClicked()));
}

TundraPreview::~TundraPreview() {

}

void TundraPreview::showEvent(QShowEvent*) {
    canvas_group->sendRepaint(false);
}

void TundraPreview::svgaClicked() {
    canvas_group->setFrameShape(QFrame::WinPanel);
    canvas_group->setFrameShadow(QFrame::Sunken);
    canvas_group->setMinimumSize(100,400);
    canvas_group->setFixedWidth(100);
    canvas_group->resizeCanvas(80);
}

void TundraPreview::vgaClicked() {
    canvas_group->setFrameShape(QFrame::WinPanel);
    canvas_group->setFrameShadow(QFrame::Sunken);
    canvas_group->setMinimumSize(180,400);
    canvas_group->setFixedWidth(180);
    canvas_group->resizeCanvas(160);
}

void TundraPreview::undo() {
    command_processor->undo();
}

void TundraPreview::redo() {
    command_processor->redo();
}

void TundraPreview::resizeEvent(QResizeEvent* e) {
    QMainWindow::resizeEvent(e);
    canvas_group->forwardResize();
}
