/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRAPREVIEW_H
#define TUNDRAPREVIEW_H

#include <qaccel.h>
#include <qmainwindow.h>
#include <qnamespace.h>
#include <qhbox.h>
#include <qscrollbar.h>
#include <qtoolbar.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>

#include "tundracanvas.h"
#include "tundracanvasgroup.h"
#include "tundramodel.h"
#include "tundracommandprocessor.h"

//! The VGA/SVGA preview window
class TundraPreview : public QMainWindow {
    Q_OBJECT

public:
    //! Constructor
    /*!
        \param m A pointer to the model
        \param cp A pointer to the local command processor
    */
    TundraPreview(TundraModel* m, TundraCommandProcessor* cp, QWidget* parent, const char* name=0);

    //! Destructor
    ~TundraPreview();

    //! Standard Qt closeEvent override
    void closeEvent (QCloseEvent* e) {
        e->accept();
        emit previewHidden();
    }

    //! Standard Qt showEvent override
    void showEvent(QShowEvent*);

    //! Standard Qt resizeEvent override
    void resizeEvent(QResizeEvent* e);

public slots:
    //! Handle SVGA radio button click
    void svgaClicked();

    //! Handle VGA radio button click
    void vgaClicked();

    //! Undo a command
    void undo();

    //! Redo a command
    void redo();

private:
    TundraModel* model;
    TundraCommandProcessor* command_processor;
    TundraCanvasGroup* canvas_group;
    QRadioButton* vga_button;
    QRadioButton* svga_button;
    QButtonGroup* selection_group;

signals:
    void previewHidden();
};

#endif

