/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrasave24bitstrategy.h"

static const char header_24bit[] = "TUNDRA24";

TundraSave24BitStrategy::TundraSave24BitStrategy()
        : TundraSaveStrategy()
{
}


TundraSave24BitStrategy::~TundraSave24BitStrategy()
{
}

bool TundraSave24BitStrategy::saveFile(QString filename, TundraDoc** document, int max_y, QByteArray* ba, QPoint* top_left, QPoint* bottom_right) {
    int max_row = 0;
    int max_x = NUM_COLS;
    int start_x = 0;
    int start_y = 0;

    if(ba != NULL) {
        max_y = bottom_right->y();
        max_x = bottom_right->x();
        start_y = top_left->y();
        start_x = top_left->x();
    }

    for(int i = start_y; i < max_y; i++) {
        for(int j = start_x; j < max_x; j++) {
            if(document[i][j].character != 0) {
                max_row = i;
                break;
            }
        }
    }

    QDataStream* os = NULL;
    QFile output(filename);

    if(!filename.isNull()) {
        output.setName(filename);
        if(output.open(IO_WriteOnly | IO_Truncate))
            os = new QDataStream(&output);
    }
    else {
        os = new QDataStream(*ba, IO_WriteOnly);
        // Write the size of the block
        int width = max_x - start_x;
        int height = max_y - start_y;

        os->writeRawBytes((char*)(&width), 4);
        os->writeRawBytes((char*)(&height), 4);
    }
    if(os != NULL) {
        if(ba == NULL) {
            // Only write the header if we're saving a file
            (*os) << (Q_INT8)24;
            for(unsigned i = 0; i < strlen(header_24bit); i++) {
                (*os) << (Q_INT8)header_24bit[i];
            }
        }

        bool blank = true;
        int fg_color = 0;
        int bg_color = 0;

        for(int i = start_y; i <= max_row; i++) {
            for(int j = start_x; j < max_x; j++) {
                char color_notice = 0;
                if(document[i][j].character != 0) {
                    if(blank == true) {
                        // Write the location
                        (*os) << (Q_INT8)1;
                        (*os) << (Q_INT32)(i-start_y);
                        (*os) << (Q_INT32)(j-start_x);

                        if(fg_color != document[i][j].fg_color) {
                            color_notice |= 2;
                            fg_color = document[i][j].fg_color;
                        }
                        if(bg_color != document[i][j].bg_color) {
                            color_notice |= 4;
                            bg_color = document[i][j].bg_color;
                        }

                        if(color_notice != 0)
                            (*os) << (Q_INT8)color_notice;

                        (*os) << (Q_INT8)document[i][j].character;

                        if(color_notice & 2)
                            (*os) << document[i][j].fg_color;
                        if(color_notice & 4)
                            (*os) << document[i][j].bg_color;

                        blank = false;
                    }
                    else {
                        // Write the character
                        if(fg_color != document[i][j].fg_color) {
                            color_notice |= 2;
                            fg_color = document[i][j].fg_color;
                        }
                        if(bg_color != document[i][j].bg_color) {
                            color_notice |= 4;
                            bg_color = document[i][j].bg_color;
                        }

                        if(color_notice != 0)
                            (*os) << (Q_INT8)color_notice;

                        (*os) << (Q_INT8)document[i][j].character;
                        if(color_notice & 2)
                            (*os) << document[i][j].fg_color;
                        if(color_notice & 4)
                            (*os) << document[i][j].bg_color;
                    }

                }
                else
                    blank = true;
            }
        }

        if(!filename.isNull()) {
            output.close();
        }

        return true;
    }

    return false;
}
