/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qfile.h>
#include <qtextstream.h>

#include "tundrasaveansistrategy.h"

TundraSaveAnsiStrategy::TundraSaveAnsiStrategy()
{
}


TundraSaveAnsiStrategy::~TundraSaveAnsiStrategy()
{
}

bool TundraSaveAnsiStrategy::saveFile(QString filename, TundraDoc** document, int max_y, QByteArray* ba, QPoint* top_left, QPoint* bottom_right) {
    // First find the last row
    int max_row = 0;
    int max_x = NUM_COLS;
    int start_x = 0;
    int start_y = 0;

    if(ba != NULL) {
        max_y = bottom_right->y();
        max_x = bottom_right->x();
        start_y = top_left->y();
        start_x = top_left->x();
    }

    for(int i = start_y; i < max_y; i++) {
        for(int j = start_x; j < max_x; j++) {
            if(document[i][j].character != 0) {
                max_row = i;
                break;
            }
        }
    }
    QFile output;

    QTextStream* os = NULL;
    if(!filename.isNull()) {
        output.setName(filename);
        if(output.open(IO_WriteOnly | IO_Truncate))
            os = new QTextStream(&output);
    }
    else {
        os = new QTextStream(*ba, IO_WriteOnly);
        // Write the size of the block
        int width = max_x - start_x;
        int height = max_y - start_y;

        os->writeRawBytes((char*)(&width), 4);
        os->writeRawBytes((char*)(&height), 4);
    }
    if(os != NULL) {
        os->setEncoding(QTextStream::Latin1);
        bool has_bg = false;
        bool has_fg = false;
        bool has_bold = false;
        bool last_row_character = false;
        unsigned char current_bg = 0;
        unsigned char current_fg = 0;
        unsigned char current_bold = 0;

        for(int i = start_y; i <= max_y; i++) {
            int over_count = 0;
            int j;

            for(j = start_x; j < max_x; j++) {

                if(document[i][j].character == 0)
                    over_count++;
                else {
                    int bold = 0;

                    if(over_count > 0) {
                        (*os) << '\033' << "[" << over_count << "C";
                        over_count = 0;
                    }

                    bool new_bold = false;
                    bool new_fg = false;
                    bool new_bg = false;
                    int fg_color = 0;

                    if(document[i][j].fg_color == 0 && !has_fg && !has_bg)
                        new_fg = false;
                    else if(document[i][j].fg_color != current_fg) {
                        if(document[i][j].fg_color > 7 && !current_bold) {
                            bold = 1;
                            new_bold = true;
                        }

                        if(document[i][j].fg_color < 7 || (document[i][j].fg_color != (current_fg + 8))) {
                            new_fg = true;
                        }
                    }

                    if(document[i][j].bg_color == 0 && !has_bg)
                        new_bg = false;
                    else if(document[i][j].bg_color != current_bg) {
                        new_bg = true;
                    }

                    // If we turn off bold, we'll need to reset everything else too
                    if(document[i][j].fg_color < 8 && current_bold) {
                        bold = 0;
                        new_bold = true;
                        new_bg = true;
                        new_fg = true;
                    }

                    if(new_bold || new_fg || new_bg)
                        (*os) << '\033' << "[";

                    if(new_bold) {
                        (*os) << bold;

                        if(!has_fg) {
                            fg_color = 0;
                            new_fg = true;
                        }

                        if(new_fg || new_bg)
                            (*os) << ";";
                        current_bold = bold;
                        has_bold = true;
                    }

                    if(new_fg) {
                        if(document[i][j].fg_color > 7)
                            fg_color = document[i][j].fg_color - 8;
                        else
                            fg_color = document[i][j].fg_color;

                        (*os) << fg_color + 30;
                        if(new_bg)
                            (*os) << ";";
                        current_fg = document[i][j].fg_color;
                        has_fg = true;
                    }

                    if(new_bg) {
                        (*os) << document[i][j].bg_color + 40;
                        current_bg = document[i][j].bg_color;
                        has_bg = true;
                    }

                    if(new_bold || new_fg || new_bg)
                        (*os) << 'm';

                    (*os) << char(document[i][j].character);
                    if(j == 79)
                        last_row_character = true;
                    else
                        last_row_character = false;
                }
            }

            if(has_bg || has_fg || has_bold) {
                if(!(current_fg == 7 && current_bg == 0 && current_bold == 0)) {
                    (*os) << '\033' << "[0m";
                    has_bold = true;
                    has_fg = true;
                    has_bg = true;
                    current_fg = 7;
                    current_bg = 0;
                    current_bold = 0;
                }
            }

            if(document[i][j-1].character == 0 && i <= max_row) {
                (*os) << "\n";
                // Special case for a blank line after the last line ended with
                // a character in the 79th column
                if(over_count == 80 && i+1 <= max_row && last_row_character == true) {
                   (*os) << "\n"; 
                    last_row_character = false;
                }
            }
        }

        if(!filename.isNull())
            output.close();

        return true;
    }
    else
        return false;
}


