/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundraserversocket.h"
#include "qmessagebox.h"

TundraServerSocket::TundraServerSocket(TundraChatWindow* cw, TundraModel* m, QString n, Q_UINT16 port, int backlog, QObject * parent, const char * name) : QServerSocket(port, backlog, parent, name), next_item(1), nick(n), model(m), chat_window(cw)
{
    users.push_back(n);
}

TundraServerSocket::~TundraServerSocket(void)
{
    QMap<TundraServerConnection*, TundraNetCommandProcessor*>::iterator processor_it;
    for(processor_it = processor_map.begin(); processor_it != processor_map.end(); processor_it++) {
        if((*processor_it) != NULL) {
            delete (*processor_it);
        }
    }

    processor_map.clear();

    QMap<TundraServerConnection*, TundraByteBuffer*>::iterator byte_buffer_it;
    for(byte_buffer_it = byte_buffer_map.begin(); byte_buffer_it != byte_buffer_map.end(); byte_buffer_it++) {
        if((*byte_buffer_it) != NULL) {
            delete (*byte_buffer_it);
        }
    }

    byte_buffer_map.clear();

    QMap<TundraServerConnection*, int>::iterator socket_it;

    for(socket_it = socket_map.begin(); socket_it != socket_map.end(); socket_it++) {
        delete socket_it.key();
    }

    socket_map.clear();
}

void TundraServerSocket::newConnection(int socket) {
    TundraServerConnection* new_socket = new TundraServerConnection();
    new_socket->setSocket(socket);

    if(deleted_ids.size() == 0) {
        socket_map[new_socket] = next_item;
        next_item++;
    }
    else {
        socket_map[new_socket] = deleted_ids.front();
        deleted_ids.pop_front();
    }

    QObject::connect(new_socket, SIGNAL(readyRead()), this, SLOT(readSocket()));
    QObject::connect(new_socket, SIGNAL(connectionClosed()), this, SLOT(clientDisconnect()));
}

void TundraServerSocket::clientDisconnect() {
    TundraServerConnection* sender = (TundraServerConnection*)(QObject::sender());

    QString name = name_map[sender];
    name_map.remove(sender);
    int name_length = name.length();

    // Remove from the user list
    int count = 0;
    QValueVector<QString>::iterator it;
    for(it = users.begin(); it != users.end(); it++) {
        if(users[count] == name) {
            users.erase(it);
            break;
        }
        count++;
    }

    // Write the logged out command to all of the nodes
    char* command = new char[name_length+2];
    command[0] = 7;
    command[1] = name_length;

    for(int i = 0; i < name_length; i++) {
        QChar ch = name.at(i);
        command[i+2] = (char)ch;
    }

    emit userLoggedOut(name);

    writeCommand(command, name_length+2);
}

void TundraServerSocket::readSocket() {
    TundraServerConnection* sender = (TundraServerConnection*)(QObject::sender());

    // Check if this socket is already logged in
    if(!sender->isLoggedIn()) {
        char name_length;
        bool login_failed = false;

        // Grab the length of the name of the person logging in
        if(sender->readBlock(&name_length, 1) != -1) {
            if(name_length > 0) {
                char* name = new char[name_length+1];
                name[name_length] = '\0';

                // Now read in their name
                if(sender->readBlock(name, name_length) != -1) {
                    emit userLoggedIn(QString(name));
                    // Successful login
                    sender->setLoggedIn();

                    processor_map[sender] = new TundraNetCommandProcessor(model);
                    byte_buffer_map[sender] = new TundraByteBuffer();

                    QObject::connect(processor_map[sender], SIGNAL(userMessage(QString,QString)), chat_window, SLOT(showUserMessage(QString,QString)));

                    // Write the logged in command to all of the nodes
                    char* command = new char[name_length+2];
                    command[0] = 6;
                    command[1] = name_length;
                    for(int i = 0; i < name_length; i++) {
                        command[i+2] = name[i];
                    }
                    writeCommand(command, name_length+2);

                    name_map[sender] = QString(name);

                    // Send the list of current users
                    for(unsigned int i = 0; i < users.size(); i++) {
                        char* user_command = new char[users[i].length()+3];
                        user_command[0] = 0;
                        user_command[1] = 6;
                        user_command[2] = (char)users[i].length();
                        for(unsigned int j = 0; j < users[i].length(); j++) {
                            QChar ch = users[i].at(j);
                            user_command[j+3] = (char)ch;
                        }

                        sender->writeBlock(user_command, users[i].length()+3);

                        delete [] user_command;
                    }

                    users.push_back(QString(name));

                    // Send the existing ANSi
                    sendAnsi(sender);

                    delete [] name;
                    delete [] command;
                }
                else
                    login_failed = true;
            }
            else {
                login_failed = true;
            }
        }
        else {
            login_failed = true;
        }

        if(login_failed) {
            sender->close();
        }
    }
    else {
        int command_length = sender->bytesAvailable();
        char* data = new char[command_length];

        // Parse the command
        sender->readBlock(data, command_length);

        Q_ULONG new_length;

        // Add the old byte buffer to the command
        data = byte_buffer_map[sender]->addAndClear(data, command_length, new_length);
        command_length = new_length;

        int bytes_processed;
        while((bytes_processed = processor_map[sender]->parseNetCommand(data, command_length)) != 0) {
            // Send to all of the other clients, prefixed with sender
            char* temp = new char[bytes_processed+1];
            temp[0] = (char)socket_map[sender];
            for(int i = 0; i < bytes_processed; i++) {
                temp[i+1] = data[i];
            }

            writeCommand(temp, (bytes_processed+1), socket_map[sender]);

            delete [] temp;

            data += bytes_processed;
            command_length -= bytes_processed;

            if(command_length == 0)
                break;
        }

        // Buffer any unprocessed data
        if(command_length > 0) {
            byte_buffer_map[sender]->add(data, command_length);
        }

    }

}

Q_ULONG TundraServerSocket::writeCommand(char* command, Q_ULONG len, int original_sender) {
    QMap<TundraServerConnection*, int>::iterator it;

    char* temp = NULL;

    if(original_sender == 0) {
        // Prefix with 0 so the client knows the server is sending this
        temp = new char[len+1];
        temp[0] = 0;
        for(unsigned int i = 0; i < len; i++) {
            temp[i+1] = command[i];
        }

        command = temp;
        len += 1;
    }

    for(it = socket_map.begin(); it != socket_map.end(); it++) {
        if(*it != original_sender) {
            it.key()->writeBlock(command, len);
        }
    }

    if(temp != NULL)
        delete [] temp;

    return 0;
}

void TundraServerSocket::closeServer() {
    QMap<TundraServerConnection*, int>::iterator it;

    for(it = socket_map.begin(); it != socket_map.end(); it++) {
        it.key()->close();
    }
}

void TundraServerSocket::sendColorMode(TundraServerConnection* connection) {
    char* command = new char[3];
    command[0] = 0;
    command[1] = 13;
    command[2] = (char)model->getColorMode();

    connection->writeBlock(command, 3);

    delete [] command;
}

void TundraServerSocket::sendColorModeToAll() {
    QMap<TundraServerConnection*, int>::iterator it;

    for(it = socket_map.begin(); it != socket_map.end(); it++) {
        sendColorMode(it.key());
    }
}

void TundraServerSocket::sendAnsi(TundraServerConnection* connection) {
    int character_count = 0;

    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            TundraDoc* doc = model->getItem(j, i);
            if(doc->character != 0)
                character_count++;
        }
    }

    char* command = new char[((character_count*17)+8)];

    command[0] = 0;
    command[1] = 0;
    unpackInteger(character_count, command+2);
    command[6] = '|';
    command[7] = (char)model->getColorMode();
    int count = 8;
    for(int i = 0; i < NUM_ROWS; i++) {
        for(int j = 0; j < NUM_COLS; j++) {
            TundraDoc* doc = model->getItem(j, i);
            if(doc != NULL) {
                if(doc->character != 0) {
                    unpackInteger(j, command+count);
                    count += 4;
                    unpackInteger(i, command+count);
                    count += 4;
                    command[count] = doc->character;
                    count++;
                    unpackInteger(doc->bg_color, command+count);
                    count += 4;
                    unpackInteger(doc->fg_color, command+count);
                    count += 4;
                }
            }
            else {
                // Something went wrong in getting an item to send
            }
        }
    }

    connection->writeBlock(command, count);
    delete [] command;
}

void TundraServerSocket::sendAnsiToAll() {
    QMap<TundraServerConnection*, int>::iterator it;

    for(it = socket_map.begin(); it != socket_map.end(); it++) {
        sendAnsi(it.key());
    }
}

void TundraServerSocket::sendTextMessage(QString message) {
    int command_length = nick.length()+5+message.length()+4;
    char* command = new char[command_length];
    command[0] = 8;
    unpackInteger(nick.length(), command+1);
    for(unsigned int i = 0; i < nick.length(); i++) {
        QChar ch = nick.at(i);
        command[i+5] = (char)ch;
    }
    unpackInteger(message.length(), command+5+nick.length());
    for(unsigned int i = 0; i < message.length(); i++) {
        QChar ch = message.at(i);
        command[i+5+nick.length()+4] = (char)ch;
    }

    writeCommand(command, command_length);
}
