/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef TUNDRASERVERSOCKET_H
#define TUNDRASERVERSOCKET_H

#include <qserversocket.h>
#include <qsocket.h>
#include <qmap.h>
#include <qvaluevector.h>

#include "tundrabytebuffer.h"
#include "tundrachatwindow.h"
#include "tundranetcommandprocessor.h"
#include "tundraserverconnection.h"
#include "tundramodel.h"
#include "utils.h"

//! The socket server
/*!
    Standard socket server for accepting incoming connections. We use non-blocking sockets, so note
    the use of the TundraByteBuffer for buffering data.

    \sa TundraNetCommandProcessor
*/
class TundraServerSocket :
            public QServerSocket
{
    Q_OBJECT
public:
    //! Constructor
    /*!
        \param cw A pointer to the chat window widget
        \param m A pointer to the model
    */
    TundraServerSocket(TundraChatWindow* cw, TundraModel* m, QString n, Q_UINT16 port, int backlog=1, QObject* parent=0, const char* name=0);
    
    //! Destructor
    ~TundraServerSocket(void);

    //! Handle a new connection
    /*!
        \param socket The socket descriptor
    */
    void newConnection(int socket);

    //! Close the server
    void closeServer();

    //! Relay a command to all of the open connections
    /*!
        \param command The command message
        \param len The length of the command
        \param original_sender The socket descriptor of the original sender so that we don't send back to the sender
    */
    Q_ULONG writeCommand(char* command, Q_ULONG len, int original_sender=0);

    //! Send the current ansi to a socket
    /*!
        \param connection The client to send to
    */
    void sendAnsi(TundraServerConnection* connection);

    //! Send the current color mode to a socket
    /*!
        \param connection The client to send to
    */
    void sendColorMode(TundraServerConnection* connection);

    //! Send the current ansi to all clients
    void sendAnsiToAll();

    //! Send the current color mode to all clients
    void sendColorModeToAll();

public slots:
    //! Read from a socket
    void readSocket();

    //! Remove a disconnected client from user lists, etc.
    void clientDisconnect();

    //! Send a text message to clients
    void sendTextMessage(QString message);

signals:
    //! Notify that a user logged in
    void userLoggedIn(QString);

    //! Notify that a user logged out
    void userLoggedOut(QString);
private:
    QMap<TundraServerConnection*, int> socket_map;
    QMap<TundraServerConnection*, TundraNetCommandProcessor*> processor_map;
    QMap<TundraServerConnection*, TundraByteBuffer*> byte_buffer_map;
    QMap<TundraServerConnection*, QString> name_map;
    int next_item;
    QValueList<int> deleted_ids;
    QString nick;
    TundraModel* model;
    TundraChatWindow* chat_window;
    QValueVector<QString> users;
};


#endif
