/***************************************************************************
 *   Copyright (C) 2003-2005 by Tom Bigwood, Ron Dicho                     *
 *   veediot@ansiscene.com, icedevil@icednet.org                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tundrastartserverdlg.h"

#include <qvariant.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

/*
 *  Constructs a TundraServerDlg as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
TundraStartServerDlg::TundraStartServerDlg( QWidget* parent, const char* name, bool modal, WFlags fl )
        : QDialog( parent, name, modal, fl )
{
    if ( !name )
        setName( "TundraStartServerDlg" );

    port_edit = new QLineEdit( this, "port_edit" );
	port_edit->setGeometry( QRect( 20, 50, 60, 21 ) );
    port_edit->setMaxLength(5);

    nick_edit = new QLineEdit( this, "nick_edit" );
    nick_edit->setGeometry( QRect( 20, 107, 161, 21 ) );
    nick_edit->setMaxLength(32);

    nick_label = new QLabel( this, "nick_label" );
	nick_label->move(20, 75);

    start_button = new QPushButton( this, "start_button" );
	start_button->move(QPoint(280,20));
	start_button->setMinimumSize(QSize(81, 31));

    start_button->setOn( FALSE );
    start_button->setDefault( TRUE );

    cancel_button = new QPushButton( this, "cancel_button" );
	cancel_button->move(280, 60);
	cancel_button->setMinimumSize(QSize(81, 31));

    port_label = new QLabel( this, "port_label" );
	port_label->move(20, 20);

/*#ifdef DEBUG
    server_opts = new QVButtonGroup(this,"server_opts");
    server_opts->setGeometry( QRect(20,120,200,100) );
    server_opts->setCaption("Server Options");
    server_readonly = new QRadioButton(server_opts);
    server_readonly->setText("Read Only");
    emit server_readonly->setChecked(TRUE);
    server_readwrite = new QRadioButton(server_opts);
    server_readwrite->setText("Allow Clients to Write");
    server_readwriteload = new QCheckBox(server_opts);
    server_readwriteload->setText("Allow Clients to Load Files");
    server_readwriteload->setEnabled(FALSE);

    read_only=true;
    allow_load=false;
#endif*/
    languageChange();
    resize( QSize(388, 255).expandedTo(minimumSizeHint()) );
    clearWState( WState_Polished );

    QObject::connect(start_button, SIGNAL(released()), this, SLOT(checkFields()));
    QObject::connect(cancel_button, SIGNAL(released()), this, SLOT(reject()));
/*#ifdef DEBUG
    QObject::connect(server_readonly,SIGNAL(released()), this,SLOT(server_readonly_Checked()));
    QObject::connect(server_readwrite,SIGNAL(released()), this,SLOT(server_readwrite_Checked()));
    QObject::connect(server_readwriteload,SIGNAL(released()), this,SLOT(server_readwriteload_Checked()));
#endif*/
}

/*
 *  Destroys the object and frees any allocated resources
 */
TundraStartServerDlg::~TundraStartServerDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void TundraStartServerDlg::languageChange()
{
    setCaption( tr( "Start Server" ) );
    nick_label->setText( tr( "Nick:" ) );
    start_button->setText( tr( "Start Server" ) );
    cancel_button->setText( tr( "Cancel" ) );
    port_label->setText( tr( "Port:" ) );
}


void TundraStartServerDlg::checkFields() {
    bool error = false;
    if(nick_edit->text().length() < 1) {
        QMessageBox::critical(this, "Error", "You must specify a nick");
        error = true;
    }
    if((port_edit->text().toInt() > 65535) || (port_edit->text().toInt() < 1)) {
        QMessageBox::critical(this, "Error", "Port number must be between 1 and 65535");
        error = true;
    }

    if(!error) {
        accept();
    }
}

void TundraStartServerDlg::server_readonly_Checked(){
    server_readwriteload->setEnabled( FALSE );
    read_only=true;
}

void TundraStartServerDlg::server_readwrite_Checked(){
    server_readwriteload->setEnabled( TRUE );
    read_only=false;

}

void TundraStartServerDlg::server_readwriteload_Checked(){
    allow_load = server_readwriteload->isChecked();
}
